var _regeneratorRuntime = require("@babel/runtime/regenerator");
var _asyncToGenerator = require("@babel/runtime/helpers/asyncToGenerator");
var Utils = require("./Utils");
exports.ValidatePresence = function (name, thing) {
  if (!thing) {
    throw Error("".concat(name, " not specified"));
  }
};
exports.ValidateLibrary = function (libraryId) {
  if (!libraryId) {
    throw Error("Library ID not specified");
  } else if (!libraryId.toString().startsWith("i")) {
    throw Error("Invalid library ID: ".concat(libraryId));
  }
};
exports.ValidateObject = function (objectId) {
  if (!objectId) {
    throw Error("Object ID not specified");
  } else if (!objectId.toString().startsWith("i")) {
    throw Error("Invalid object ID: ".concat(objectId));
  }
};
exports.ValidateVersion = function (versionHash) {
  if (!versionHash) {
    throw Error("Version hash not specified");
  } else if (!versionHash.toString().startsWith("h")) {
    throw Error("Invalid version hash: ".concat(versionHash));
  }
};
exports.ValidateWriteToken = function (writeToken) {
  if (!writeToken) {
    throw Error("Write token not specified");
  } else if (!writeToken.toString().startsWith("t")) {
    throw Error("Invalid write token: ".concat(writeToken));
  }
};
exports.ValidatePartHash = function (partHash) {
  if (!partHash) {
    throw Error("Part hash not specified");
  } else if (!partHash.toString().startsWith("hqp_") && !partHash.toString().startsWith("hqpe") && !partHash.toString().startsWith("hqt_") && !partHash.toString().startsWith("hqte") && !partHash.toString().startsWith("hql_") && !partHash.toString().startsWith("hqle")) {
    throw Error("Invalid part hash: ".concat(partHash));
  }
};
exports.ValidateParameters = function (_ref) {
  var libraryId = _ref.libraryId,
    objectId = _ref.objectId,
    versionHash = _ref.versionHash,
    writeToken = _ref.writeToken;
  if (writeToken) {
    if (versionHash) throw Error("Cannot specify writeToken and versionHash at same time (token:".concat(writeToken, ", hash:").concat(versionHash, ")"));
    exports.ValidateWriteToken(writeToken);
  } else if (versionHash) {
    exports.ValidateVersion(versionHash);
  } else {
    exports.ValidateLibrary(libraryId);
    exports.ValidateObject(objectId);
  }
};
exports.ValidateAddress = function (address) {
  if (!address) {
    throw Error("Address not specified");
  } else if (!/^(0x)?[0-9a-f]{40}$/i.test(address.toLowerCase())) {
    throw Error("Invalid address: ".concat(address));
  }
  return Utils.FormatAddress(address);
};
exports.ValidatePermission = function (permission) {
  if (permission && permission !== "full-access" && permission !== "no-access") {
    throw Error("Invalid profile permission: ".concat(permission));
  }
  return permission;
};
exports.ValidateDate = function (date) {
  if (!date) {
    return;
  }
  if (isNaN(new Date(date))) {
    throw Error("Invalid date: ".concat(date));
  }
  return new Date(date).getTime();
};
exports.ValidateUserWallet = /*#__PURE__*/function () {
  var _ref3 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(_ref2) {
    var address, client, walletAddress;
    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) switch (_context.prev = _context.next) {
        case 0:
          address = _ref2.address, client = _ref2.client;
          if (address) {
            _context.next = 5;
            break;
          }
          throw Error("Address not specified");
        case 5:
          if (client) {
            _context.next = 7;
            break;
          }
          throw Error("Client not specified");
        case 7:
          _context.next = 9;
          return client.userProfileClient.UserWalletAddress({
            address: address
          });
        case 9:
          walletAddress = _context.sent;
          if (walletAddress) {
            _context.next = 12;
            break;
          }
          throw Error("User address has no wallet: ".concat(address));
        case 12:
        case "end":
          return _context.stop();
      }
    }, _callee);
  }));
  return function (_x) {
    return _ref3.apply(this, arguments);
  };
}();