var _defineProperty = require("@babel/runtime/helpers/defineProperty");
var _typeof = require("@babel/runtime/helpers/typeof");
var _regeneratorRuntime = require("@babel/runtime/regenerator");
var _asyncToGenerator = require("@babel/runtime/helpers/asyncToGenerator");
function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
/**
 * Methods for deploying and interacting with contracts
 *
 * @module ElvClient/Contracts
 */

var Ethers = require("ethers");
//const ContentContract = require("../contracts/BaseContent");

var _require = require("../Validation"),
  ValidateAddress = _require.ValidateAddress,
  ValidateParameters = _require.ValidateParameters,
  ValidatePresence = _require.ValidatePresence,
  ValidateObject = _require.ValidateObject,
  ValidateVersion = _require.ValidateVersion;
var Utils = require("../Utils");

/**
 * Return the name of the contract, as specified in the contracts "version" string
 *
 * @methodGroup Contracts
 *
 * @namedParams
 * @param {string} contractAddress - Address of the contract
 *
 * @return {Promise<string>} - Name of the contract
 */
exports.ContractName = /*#__PURE__*/function () {
  var _ref2 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(_ref) {
    var contractAddress;
    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) switch (_context.prev = _context.next) {
        case 0:
          contractAddress = _ref.contractAddress;
          contractAddress = ValidateAddress(contractAddress);
          _context.next = 4;
          return this.ethClient.ContractName(contractAddress);
        case 4:
          return _context.abrupt("return", _context.sent);
        case 5:
        case "end":
          return _context.stop();
      }
    }, _callee, this);
  }));
  return function (_x) {
    return _ref2.apply(this, arguments);
  };
}();

/**
 * Retrieve the ABI for the given contract via its address or a Fabric ID. Contract must be a standard Eluvio contract
 *
 * @methodGroup Contracts
 * @namedParams
 * @param {string=} contractAddress - The address of the contract
 * @param {string=} id - The Fabric ID of the contract
 *
 * @return {Promise<Object>} - The ABI for the given contract
 *
 * @throws If ABI is not able to be determined, throws an error
 */
exports.ContractAbi = /*#__PURE__*/function () {
  var _ref4 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2(_ref3) {
    var contractAddress, id, contractInfo;
    return _regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) switch (_context2.prev = _context2.next) {
        case 0:
          contractAddress = _ref3.contractAddress, id = _ref3.id;
          _context2.next = 3;
          return this.authClient.ContractInfo({
            address: contractAddress,
            id: id
          });
        case 3:
          contractInfo = _context2.sent;
          if (contractInfo) {
            _context2.next = 6;
            break;
          }
          throw Error("Unable to determine contract info for ".concat(contractAddress));
        case 6:
          return _context2.abrupt("return", contractInfo.abi);
        case 7:
        case "end":
          return _context2.stop();
      }
    }, _callee2, this);
  }));
  return function (_x2) {
    return _ref4.apply(this, arguments);
  };
}();

/**
 * Retrieve the ABI, access type, and whether V3 is used for a given contract via its address or a Fabric ID.
 *
 * @methodGroup Contracts
 * @namedParams
 * @param {string=} id - The Fabric ID of the contract
 * @param {string=} address - The address of the contract
 *
 * @return {Promise<Object>} - The ABI, access type, and isV3 for the given contract
 */
exports.ContractInfo = /*#__PURE__*/function () {
  var _ref6 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee3(_ref5) {
    var id, address;
    return _regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) switch (_context3.prev = _context3.next) {
        case 0:
          id = _ref5.id, address = _ref5.address;
          return _context3.abrupt("return", this.authClient.ContractInfo({
            id: id,
            address: address
          }));
        case 2:
        case "end":
          return _context3.stop();
      }
    }, _callee3, this);
  }));
  return function (_x3) {
    return _ref6.apply(this, arguments);
  };
}();

/**
 * Format the arguments to be used for the specified method of the contract
 *
 * @methodGroup Contracts
 * @namedParams
 * @param {Object} abi - ABI of contract
 * @param {string} methodName - Name of method for which arguments will be formatted
 * @param {Array<string>} args - List of arguments
 *
 * @returns {Array<string>} - List of formatted arguments
 */
exports.FormatContractArguments = function (_ref7) {
  var abi = _ref7.abi,
    methodName = _ref7.methodName,
    args = _ref7.args;
  return this.ethClient.FormatContractArguments({
    abi: abi,
    methodName: methodName,
    args: args
  });
};

/**
 * Deploy a contract from ABI and bytecode. This client's signer will be the owner of the contract.
 *
 * @methodGroup Contracts
 * @namedParams
 * @param {Object} abi - ABI of contract
 * @param {string} bytecode - Bytecode of the contract
 * @param {Array<string>} constructorArgs - List of arguments to the contract constructor
 * @param {Object=} overrides - Change default gasPrice or gasLimit used for this action
 *
 * @returns {Promise<Object>} - Response containing the deployed contract address and the transaction hash of the deployment
 */
exports.DeployContract = /*#__PURE__*/function () {
  var _ref9 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee4(_ref8) {
    var abi, bytecode, constructorArgs, _ref8$overrides, overrides;
    return _regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) switch (_context4.prev = _context4.next) {
        case 0:
          abi = _ref8.abi, bytecode = _ref8.bytecode, constructorArgs = _ref8.constructorArgs, _ref8$overrides = _ref8.overrides, overrides = _ref8$overrides === void 0 ? {} : _ref8$overrides;
          _context4.next = 3;
          return this.ethClient.DeployContract({
            abi: abi,
            bytecode: bytecode,
            constructorArgs: constructorArgs,
            overrides: overrides,
            signer: this.signer
          });
        case 3:
          return _context4.abrupt("return", _context4.sent);
        case 4:
        case "end":
          return _context4.stop();
      }
    }, _callee4, this);
  }));
  return function (_x4) {
    return _ref9.apply(this, arguments);
  };
}();

/**
 * Get all events on the specified contract
 *
 * @methodGroup Contracts
 * @namedParams
 * @param {string} contractAddress - The address of the contract
 * @param {Object=} abi - ABI of contract - If the contract is a standard Eluvio contract, this can be determined automatically if not specified
 * @param {number=} fromBlock - Limit results to events after the specified block (inclusive)
 * @param {number=} toBlock - Limit results to events before the specified block (inclusive)
 * @param {number=} count=1000 - Maximum range of blocks to search (unless both toBlock and fromBlock are specified)
 * @param {boolean=} includeTransaction=false - If specified, more detailed transaction info will be included.
 * Note: This requires one extra network call per block, so it should not be used for very large ranges
 * @returns {Promise<Array<Array<Object>>>} - List of blocks, in ascending order by block number, each containing a list of the events in the block.
 */
exports.ContractEvents = /*#__PURE__*/function () {
  var _ref11 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee5(_ref10) {
    var contractAddress, abi, _ref10$fromBlock, fromBlock, toBlock, _ref10$count, count, topics, _ref10$includeTransac, includeTransaction, blocks;
    return _regeneratorRuntime.wrap(function _callee5$(_context5) {
      while (1) switch (_context5.prev = _context5.next) {
        case 0:
          contractAddress = _ref10.contractAddress, abi = _ref10.abi, _ref10$fromBlock = _ref10.fromBlock, fromBlock = _ref10$fromBlock === void 0 ? 0 : _ref10$fromBlock, toBlock = _ref10.toBlock, _ref10$count = _ref10.count, count = _ref10$count === void 0 ? 1000 : _ref10$count, topics = _ref10.topics, _ref10$includeTransac = _ref10.includeTransaction, includeTransaction = _ref10$includeTransac === void 0 ? false : _ref10$includeTransac;
          contractAddress = ValidateAddress(contractAddress);
          if (abi) {
            _context5.next = 6;
            break;
          }
          _context5.next = 5;
          return this.ContractAbi({
            contractAddress: contractAddress
          });
        case 5:
          abi = _context5.sent;
        case 6:
          _context5.next = 8;
          return this.FormatBlockNumbers({
            fromBlock: fromBlock,
            toBlock: toBlock,
            count: count
          });
        case 8:
          blocks = _context5.sent;
          this.Log("Querying contract events ".concat(contractAddress, " - Blocks ").concat(blocks.fromBlock, " to ").concat(blocks.toBlock));
          _context5.next = 12;
          return this.ethClient.ContractEvents({
            contractAddress: contractAddress,
            abi: abi,
            fromBlock: blocks.fromBlock,
            toBlock: blocks.toBlock,
            topics: topics,
            includeTransaction: includeTransaction
          });
        case 12:
          return _context5.abrupt("return", _context5.sent);
        case 13:
        case "end":
          return _context5.stop();
      }
    }, _callee5, this);
  }));
  return function (_x5) {
    return _ref11.apply(this, arguments);
  };
}();

/**
 * Call the specified method on a deployed contract. This action will be performed by this client's signer.
 *
 * Use this method to call constant methods and contract attributes, as well as transaction-performing methods
 * for which the transaction does not need to be awaited.
 *
 * @methodGroup Contracts
 * @namedParams
 * @param {string} contractAddress - Address of the contract to call the specified method on
 * @param {Object=} abi - ABI of contract - If the contract is a standard Eluvio contract, this can be determined automatically if not specified
 * @param {string} methodName - Method to call on the contract
 * @param {Array=} methodArgs - List of arguments to the contract constructor
 * @param {(number | BigNumber)=} value - Amount of ether to include in the transaction
 * @param {boolean=} formatArguments=true - If specified, the arguments will automatically be formatted to the ABI specification
 * @param {Object=} overrides - Change default gasPrice or gasLimit used for this action
 *
 * @returns {Promise<*>} - Response containing information about the transaction
 */
exports.CallContractMethod = /*#__PURE__*/function () {
  var _ref13 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee6(_ref12) {
    var contractAddress, abi, methodName, _ref12$methodArgs, methodArgs, value, _ref12$overrides, overrides, _ref12$formatArgument, formatArguments, _ref12$cacheContract, cacheContract, _ref12$overrideCached, overrideCachedContract;
    return _regeneratorRuntime.wrap(function _callee6$(_context6) {
      while (1) switch (_context6.prev = _context6.next) {
        case 0:
          contractAddress = _ref12.contractAddress, abi = _ref12.abi, methodName = _ref12.methodName, _ref12$methodArgs = _ref12.methodArgs, methodArgs = _ref12$methodArgs === void 0 ? [] : _ref12$methodArgs, value = _ref12.value, _ref12$overrides = _ref12.overrides, overrides = _ref12$overrides === void 0 ? {} : _ref12$overrides, _ref12$formatArgument = _ref12.formatArguments, formatArguments = _ref12$formatArgument === void 0 ? true : _ref12$formatArgument, _ref12$cacheContract = _ref12.cacheContract, cacheContract = _ref12$cacheContract === void 0 ? true : _ref12$cacheContract, _ref12$overrideCached = _ref12.overrideCachedContract, overrideCachedContract = _ref12$overrideCached === void 0 ? false : _ref12$overrideCached;
          contractAddress = ValidateAddress(contractAddress);

          // Delete cached visibility value if it is being changed
          contractAddress = this.utils.FormatAddress(contractAddress);
          if (methodName === "setVisibility" && this.visibilityInfo[contractAddress]) {
            delete this.visibilityInfo[contractAddress];
          }
          if (abi) {
            _context6.next = 8;
            break;
          }
          _context6.next = 7;
          return this.ContractAbi({
            contractAddress: contractAddress
          });
        case 7:
          abi = _context6.sent;
        case 8:
          _context6.next = 10;
          return this.ethClient.CallContractMethod({
            contractAddress: contractAddress,
            abi: abi,
            methodName: methodName,
            methodArgs: methodArgs,
            value: value,
            overrides: overrides,
            formatArguments: formatArguments,
            cacheContract: cacheContract,
            overrideCachedContract: overrideCachedContract
          });
        case 10:
          return _context6.abrupt("return", _context6.sent);
        case 11:
        case "end":
          return _context6.stop();
      }
    }, _callee6, this);
  }));
  return function (_x6) {
    return _ref13.apply(this, arguments);
  };
}();

/**
 * Call the specified method on a deployed contract and wait for the transaction to be mined.
 * This action will be performed by this client's signer.
 *
 * Use this method to call transaction-performing methods and wait for the transaction to complete.
 *
 * @methodGroup Contracts
 * @namedParams
 * @param {string} contractAddress - Address of the contract to call the specified method on
 * @param {Object=} abi - ABI of contract - If the contract is a standard Eluvio contract, this can be determined automatically if not specified
 * @param {string} methodName - Method to call on the contract
 * @param {Array<string>=} methodArgs=[] - List of arguments to the contract constructor
 * @param {(number | BigNumber)=} value - Amount of ether to include in the transaction
 * @param {Object=} overrides - Change default gasPrice or gasLimit used for this action
 * @param {boolean=} formatArguments=true - If specified, the arguments will automatically be formatted to the ABI specification
 *
 * @see Utils.WeiToEther
 *
 * @returns {Promise<*>} - The event object of this transaction. See the ExtractEventFromLogs method for parsing
 * the resulting event(s)
 */
exports.CallContractMethodAndWait = /*#__PURE__*/function () {
  var _ref15 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee7(_ref14) {
    var contractAddress, abi, methodName, methodArgs, value, _ref14$overrides, overrides, _ref14$formatArgument, formatArguments, _ref14$cacheContract, cacheContract, _ref14$overrideCached, overrideCachedContract;
    return _regeneratorRuntime.wrap(function _callee7$(_context7) {
      while (1) switch (_context7.prev = _context7.next) {
        case 0:
          contractAddress = _ref14.contractAddress, abi = _ref14.abi, methodName = _ref14.methodName, methodArgs = _ref14.methodArgs, value = _ref14.value, _ref14$overrides = _ref14.overrides, overrides = _ref14$overrides === void 0 ? {} : _ref14$overrides, _ref14$formatArgument = _ref14.formatArguments, formatArguments = _ref14$formatArgument === void 0 ? true : _ref14$formatArgument, _ref14$cacheContract = _ref14.cacheContract, cacheContract = _ref14$cacheContract === void 0 ? true : _ref14$cacheContract, _ref14$overrideCached = _ref14.overrideCachedContract, overrideCachedContract = _ref14$overrideCached === void 0 ? false : _ref14$overrideCached;
          contractAddress = ValidateAddress(contractAddress);

          // Delete cached visibility value if it is being changed
          contractAddress = this.utils.FormatAddress(contractAddress);
          if (methodName === "setVisibility" && this.visibilityInfo[contractAddress]) {
            delete this.visibilityInfo[contractAddress];
          }
          if (abi) {
            _context7.next = 8;
            break;
          }
          _context7.next = 7;
          return this.ContractAbi({
            contractAddress: contractAddress
          });
        case 7:
          abi = _context7.sent;
        case 8:
          _context7.next = 10;
          return this.ethClient.CallContractMethodAndWait({
            contractAddress: contractAddress,
            abi: abi,
            methodName: methodName,
            methodArgs: methodArgs,
            value: value,
            overrides: overrides,
            formatArguments: formatArguments,
            cacheContract: cacheContract,
            overrideCachedContract: overrideCachedContract
          });
        case 10:
          return _context7.abrupt("return", _context7.sent);
        case 11:
        case "end":
          return _context7.stop();
      }
    }, _callee7, this);
  }));
  return function (_x7) {
    return _ref15.apply(this, arguments);
  };
}();

/**
 * Retrieve metadata from the specified contract
 *
 * @methodGroup Contracts
 * @namedParams
 * @param {string} contractAddress - The address of the contract
 * @param {string} metadataKey - The metadata key to retrieve
 *
 * @return {Promise<Object|string>}
 */
exports.ContractMetadata = /*#__PURE__*/function () {
  var _ref17 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee8(_ref16) {
    var contractAddress, metadataKey, metadata, data;
    return _regeneratorRuntime.wrap(function _callee8$(_context8) {
      while (1) switch (_context8.prev = _context8.next) {
        case 0:
          contractAddress = _ref16.contractAddress, metadataKey = _ref16.metadataKey;
          ValidatePresence("contractAddress", contractAddress);
          ValidatePresence("metadataKey", metadataKey);
          _context8.prev = 3;
          _context8.next = 6;
          return this.CallContractMethod({
            contractAddress: contractAddress,
            methodName: "getMeta",
            methodArgs: [metadataKey]
          });
        case 6:
          metadata = _context8.sent;
          data = Buffer.from((metadata || "").replace("0x", ""), "hex").toString("utf-8");
          _context8.prev = 8;
          return _context8.abrupt("return", JSON.parse(data));
        case 12:
          _context8.prev = 12;
          _context8.t0 = _context8["catch"](8);
          return _context8.abrupt("return", data);
        case 15:
          _context8.next = 20;
          break;
        case 17:
          _context8.prev = 17;
          _context8.t1 = _context8["catch"](3);
          return _context8.abrupt("return", "");
        case 20:
        case "end":
          return _context8.stop();
      }
    }, _callee8, this, [[3, 17], [8, 12]]);
  }));
  return function (_x8) {
    return _ref17.apply(this, arguments);
  };
}();

/**
 * Merge contract metadata at the specified key.
 *
 * @methodGroup Contracts
 * @namedParams
 * @param {string} contractAddress - The address of the contract
 * @param {string} metadataKey - The metadata key to retrieve
 * @param {string} metadata
 */
exports.MergeContractMetadata = /*#__PURE__*/function () {
  var _ref19 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee9(_ref18) {
    var contractAddress, metadataKey, metadata, existingMetadata;
    return _regeneratorRuntime.wrap(function _callee9$(_context9) {
      while (1) switch (_context9.prev = _context9.next) {
        case 0:
          contractAddress = _ref18.contractAddress, metadataKey = _ref18.metadataKey, metadata = _ref18.metadata;
          ValidatePresence("contractAddress", contractAddress);
          ValidatePresence("metadataKey", metadataKey);
          _context9.next = 5;
          return this.ContractMetadata({
            contractAddress: contractAddress,
            metadataKey: metadataKey
          });
        case 5:
          _context9.t0 = _context9.sent;
          if (_context9.t0) {
            _context9.next = 8;
            break;
          }
          _context9.t0 = {};
        case 8:
          existingMetadata = _context9.t0;
          if (_typeof(existingMetadata) === "object") {
            metadata = _objectSpread(_objectSpread({}, existingMetadata), metadata);
          }
          _context9.next = 12;
          return this.CallContractMethodAndWait({
            contractAddress: contractAddress,
            methodName: "putMeta",
            methodArgs: [metadataKey, JSON.stringify(metadata)]
          });
        case 12:
        case "end":
          return _context9.stop();
      }
    }, _callee9, this);
  }));
  return function (_x9) {
    return _ref19.apply(this, arguments);
  };
}();

/**
 * Replace the contract metadata at the specified key
 *
 * @methodGroup Contracts
 * @namedParams
 * @param {string} contractAddress - The address of the contract
 * @param {string} metadataKey - The metadata key to retrieve
 * @param {string|Object} metadata - The metadata to insert
 */
exports.ReplaceContractMetadata = /*#__PURE__*/function () {
  var _ref21 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee10(_ref20) {
    var contractAddress, metadataKey, metadata;
    return _regeneratorRuntime.wrap(function _callee10$(_context10) {
      while (1) switch (_context10.prev = _context10.next) {
        case 0:
          contractAddress = _ref20.contractAddress, metadataKey = _ref20.metadataKey, metadata = _ref20.metadata;
          ValidatePresence("contractAddress", contractAddress);
          ValidatePresence("metadataKey", metadataKey);
          if (_typeof(metadata) === "object") {
            metadata = JSON.stringify(metadata);
          }
          _context10.next = 6;
          return this.CallContractMethodAndWait({
            contractAddress: contractAddress,
            methodName: "putMeta",
            methodArgs: [metadataKey, metadata]
          });
        case 6:
        case "end":
          return _context10.stop();
      }
    }, _callee10, this);
  }));
  return function (_x10) {
    return _ref21.apply(this, arguments);
  };
}();

/**
 * Get the custom contract of the specified object
 *
 * @methodGroup Contracts
 * @namedParams
 * @param {string=} libraryId - ID of the library
 * @param {string=} objectId - ID of the object
 * @param {string=} versionHash - Version hash of the object
 *
 * @returns {Promise<string> | undefined} - If the object has a custom contract, this will return the address of the custom contract
 */
exports.CustomContractAddress = /*#__PURE__*/function () {
  var _ref23 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee11(_ref22) {
    var libraryId, objectId, versionHash, abi, customContractAddress;
    return _regeneratorRuntime.wrap(function _callee11$(_context11) {
      while (1) switch (_context11.prev = _context11.next) {
        case 0:
          libraryId = _ref22.libraryId, objectId = _ref22.objectId, versionHash = _ref22.versionHash;
          ValidateParameters({
            libraryId: libraryId,
            objectId: objectId,
            versionHash: versionHash
          });
          if (versionHash) {
            objectId = this.utils.DecodeVersionHash(versionHash).objectId;
          }
          if (!(libraryId === this.contentSpaceLibraryId || this.utils.EqualHash(libraryId, objectId))) {
            _context11.next = 5;
            break;
          }
          return _context11.abrupt("return");
        case 5:
          this.Log("Retrieving custom contract address: ".concat(objectId));
          _context11.next = 8;
          return this.ContractAbi({
            id: objectId
          });
        case 8:
          abi = _context11.sent;
          _context11.next = 11;
          return this.ethClient.CallContractMethod({
            contractAddress: this.utils.HashToAddress(objectId),
            abi: abi,
            methodName: "contentContractAddress",
            methodArgs: []
          });
        case 11:
          customContractAddress = _context11.sent;
          if (!(customContractAddress === this.utils.nullAddress)) {
            _context11.next = 14;
            break;
          }
          return _context11.abrupt("return");
        case 14:
          return _context11.abrupt("return", this.utils.FormatAddress(customContractAddress));
        case 15:
        case "end":
          return _context11.stop();
      }
    }, _callee11, this);
  }));
  return function (_x11) {
    return _ref23.apply(this, arguments);
  };
}();

/**
 * Set the custom contract of the specified object with the contract at the specified address
 *
 * Note: This also updates the content object metadata with information about the contract - particularly the ABI
 *
 * @methodGroup Contracts
 * @namedParams
 * @param {string} libraryId - ID of the library
 * @param {string} objectId - ID of the object
 * @param {string} customContractAddress - Address of the deployed custom contract
 * @param {string=} name - Optional name of the custom contract
 * @param {string=} description - Optional description of the custom contract
 * @param {Object} abi - ABI of the custom contract
 * @param {Object=} factoryAbi - If the custom contract is a factory, the ABI of the contract it deploys
 * @param {Object=} overrides - Change default gasPrice or gasLimit used for this action
 *
 * @returns {Promise<Object>} - Result transaction of calling the setCustomContract method on the content object contract
 */
exports.SetCustomContentContract = /*#__PURE__*/function () {
  var _ref25 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee12(_ref24) {
    var libraryId, objectId, customContractAddress, name, description, abi, factoryAbi, _ref24$overrides, overrides, setResult, writeToken;
    return _regeneratorRuntime.wrap(function _callee12$(_context12) {
      while (1) switch (_context12.prev = _context12.next) {
        case 0:
          libraryId = _ref24.libraryId, objectId = _ref24.objectId, customContractAddress = _ref24.customContractAddress, name = _ref24.name, description = _ref24.description, abi = _ref24.abi, factoryAbi = _ref24.factoryAbi, _ref24$overrides = _ref24.overrides, overrides = _ref24$overrides === void 0 ? {} : _ref24$overrides;
          ValidateParameters({
            libraryId: libraryId,
            objectId: objectId
          });
          customContractAddress = ValidateAddress(customContractAddress);
          customContractAddress = this.utils.FormatAddress(customContractAddress);
          this.Log("Setting custom contract address: ".concat(objectId, " ").concat(customContractAddress));
          _context12.next = 7;
          return this.ethClient.SetCustomContentContract({
            contentContractAddress: this.utils.HashToAddress(objectId),
            customContractAddress: customContractAddress,
            overrides: overrides,
            signer: this.signer
          });
        case 7:
          setResult = _context12.sent;
          _context12.next = 10;
          return this.EditContentObject({
            libraryId: libraryId,
            objectId: objectId
          });
        case 10:
          writeToken = _context12.sent.write_token;
          _context12.next = 13;
          return this.ReplaceMetadata({
            libraryId: libraryId,
            objectId: objectId,
            writeToken: writeToken,
            metadataSubtree: "custom_contract",
            metadata: {
              name: name,
              description: description,
              address: customContractAddress,
              abi: abi,
              factoryAbi: factoryAbi
            }
          });
        case 13:
          _context12.next = 15;
          return this.FinalizeContentObject({
            libraryId: libraryId,
            objectId: objectId,
            writeToken: writeToken,
            commitMessage: "Set custom contract"
          });
        case 15:
          return _context12.abrupt("return", setResult);
        case 16:
        case "end":
          return _context12.stop();
      }
    }, _callee12, this);
  }));
  return function (_x12) {
    return _ref25.apply(this, arguments);
  };
}();

/**
 * Extract the specified event log from the given event obtained from the
 * CallContractAndMethodAndWait method
 *
 * @methodGroup Contracts
 * @namedParams
 * @param {string} contractAddress - Address of the contract to call the specified method on
 *
 * @param {Object} event - Event of the transaction from CallContractMethodAndWait
 * @param {string} eventName - Name of the event to parse
 *
 * @see Utils.WeiToEther
 *
 * @returns {Promise<Object>} - The parsed event log from the event
 */
exports.ExtractEventFromLogs = function (_ref26) {
  var abi = _ref26.abi,
    event = _ref26.event,
    eventName = _ref26.eventName;
  return this.ethClient.ExtractEventFromLogs({
    abi: abi,
    event: event,
    eventName: eventName
  });
};

/**
 * Extract the specified value from the specified event log from the given event obtained
 * from the CallContractAndMethodAndWait method
 *
 * @methodGroup Contracts
 * @namedParams
 * @param {string} contractAddress - Address of the contract to call the specified method on
 * @param {Object} abi - ABI of contract
 * @param {Object} event - Event of the transaction from CallContractMethodAndWait
 * @param {string} eventName - Name of the event to parse
 * @param {string} eventValue - Name of the value to extract from the event
 *
 * @returns {Promise<string>} The value extracted from the event
 */
exports.ExtractValueFromEvent = function (_ref27) {
  var abi = _ref27.abi,
    event = _ref27.event,
    eventName = _ref27.eventName,
    eventValue = _ref27.eventValue;
  var eventLog = this.ethClient.ExtractEventFromLogs({
    abi: abi,
    event: event,
    eventName: eventName,
    eventValue: eventValue
  });
  return eventLog ? eventLog.args[eventValue] : undefined;
};
exports.FormatBlockNumbers = /*#__PURE__*/function () {
  var _ref29 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee13(_ref28) {
    var fromBlock, toBlock, _ref28$count, count, latestBlock;
    return _regeneratorRuntime.wrap(function _callee13$(_context13) {
      while (1) switch (_context13.prev = _context13.next) {
        case 0:
          fromBlock = _ref28.fromBlock, toBlock = _ref28.toBlock, _ref28$count = _ref28.count, count = _ref28$count === void 0 ? 10 : _ref28$count;
          _context13.next = 3;
          return this.BlockNumber();
        case 3:
          latestBlock = _context13.sent;
          if (!toBlock) {
            if (!fromBlock) {
              toBlock = latestBlock;
              fromBlock = toBlock - count + 1;
            } else {
              toBlock = fromBlock + count - 1;
            }
          } else if (!fromBlock) {
            fromBlock = toBlock - count + 1;
          }

          // Ensure block numbers are valid
          if (toBlock > latestBlock) {
            toBlock = latestBlock;
          }
          if (fromBlock < 0) {
            fromBlock = 0;
          }
          return _context13.abrupt("return", {
            fromBlock: fromBlock,
            toBlock: toBlock
          });
        case 8:
        case "end":
          return _context13.stop();
      }
    }, _callee13, this);
  }));
  return function (_x13) {
    return _ref29.apply(this, arguments);
  };
}();

/**
 * Get events from the blockchain in reverse chronological order, starting from toBlock. This will also attempt
 * to identify and parse any known Eluvio contract methods. If successful, the method name, signature, and input
 * values will be included in the log entry.
 *
 * @methodGroup Blockchain
 * @namedParams
 * @param {number=} toBlock - Limit results to events before the specified block (inclusive) - If not specified, will start from latest block
 * @param {number=} fromBlock - Limit results to events after the specified block (inclusive)
 * @param {number=} count=10 - Max number of events to include (unless both toBlock and fromBlock are specified)
 * @param {boolean=} includeTransaction=false - If specified, more detailed transaction info will be included.
 * Note: This requires two extra network calls per transaction, so it should not be used for very large ranges
 * @returns {Promise<Array<Array<Object>>>} - List of blocks, in ascending order by block number, each containing a list of the events in the block.
 */
exports.Events = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee14() {
  var _ref31,
    toBlock,
    fromBlock,
    _ref31$count,
    count,
    _ref31$includeTransac,
    includeTransaction,
    blocks,
    _args14 = arguments;
  return _regeneratorRuntime.wrap(function _callee14$(_context14) {
    while (1) switch (_context14.prev = _context14.next) {
      case 0:
        _ref31 = _args14.length > 0 && _args14[0] !== undefined ? _args14[0] : {}, toBlock = _ref31.toBlock, fromBlock = _ref31.fromBlock, _ref31$count = _ref31.count, count = _ref31$count === void 0 ? 10 : _ref31$count, _ref31$includeTransac = _ref31.includeTransaction, includeTransaction = _ref31$includeTransac === void 0 ? false : _ref31$includeTransac;
        _context14.next = 3;
        return this.FormatBlockNumbers({
          fromBlock: fromBlock,
          toBlock: toBlock,
          count: count
        });
      case 3:
        blocks = _context14.sent;
        this.Log("Querying events - Blocks ".concat(blocks.fromBlock, " to ").concat(blocks.toBlock));
        _context14.next = 7;
        return this.ethClient.Events({
          fromBlock: blocks.fromBlock,
          toBlock: blocks.toBlock,
          includeTransaction: includeTransaction
        });
      case 7:
        return _context14.abrupt("return", _context14.sent);
      case 8:
      case "end":
        return _context14.stop();
    }
  }, _callee14, this);
}));

/**
 * Retrieve the latest block number on the blockchain
 *
 * @methodGroup Blockchain
 *
 * @returns {Promise<number>} - The latest block number
 */
exports.BlockNumber = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee15() {
  return _regeneratorRuntime.wrap(function _callee15$(_context15) {
    while (1) switch (_context15.prev = _context15.next) {
      case 0:
        _context15.next = 2;
        return this.ethClient.MakeProviderCall({
          methodName: "getBlockNumber"
        });
      case 2:
        return _context15.abrupt("return", _context15.sent);
      case 3:
      case "end":
        return _context15.stop();
    }
  }, _callee15, this);
}));

/**
 * Get the balance (in ether) of the specified address
 *
 * @methodGroup Blockchain
 * @namedParams
 * @param {string} address - Address to query
 *
 * @returns {Promise<string>} - Balance of the account, in ether (as string)
 */
exports.GetBalance = /*#__PURE__*/function () {
  var _ref34 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee16(_ref33) {
    var address, balance;
    return _regeneratorRuntime.wrap(function _callee16$(_context16) {
      while (1) switch (_context16.prev = _context16.next) {
        case 0:
          address = _ref33.address;
          address = ValidateAddress(address);
          _context16.next = 4;
          return this.ethClient.MakeProviderCall({
            methodName: "getBalance",
            args: [address]
          });
        case 4:
          balance = _context16.sent;
          return _context16.abrupt("return", Ethers.utils.formatEther(balance));
        case 6:
        case "end":
          return _context16.stop();
      }
    }, _callee16, this);
  }));
  return function (_x14) {
    return _ref34.apply(this, arguments);
  };
}();

/**
 * Send ether from this client's current signer to the specified recipient address
 *
 * @methodGroup Blockchain
 * @namedParams
 * @param {string} recipient - Address of the recipient
 * @param {number} ether - Amount of ether to send
 *
 * @returns {Promise<Object>} - The transaction receipt
 */
exports.SendFunds = /*#__PURE__*/function () {
  var _ref36 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee17(_ref35) {
    var recipient, ether, transaction;
    return _regeneratorRuntime.wrap(function _callee17$(_context17) {
      while (1) switch (_context17.prev = _context17.next) {
        case 0:
          recipient = _ref35.recipient, ether = _ref35.ether;
          recipient = ValidateAddress(recipient);
          _context17.next = 4;
          return this.signer.sendTransaction({
            to: recipient,
            value: Ethers.utils.parseEther(ether.toString())
          });
        case 4:
          transaction = _context17.sent;
          _context17.next = 7;
          return transaction.wait();
        case 7:
          return _context17.abrupt("return", _context17.sent);
        case 8:
        case "end":
          return _context17.stop();
      }
    }, _callee17, this);
  }));
  return function (_x15) {
    return _ref36.apply(this, arguments);
  };
}();
var GetObjectIDAndContractAddress = /*#__PURE__*/function () {
  var _ref38 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee18(_ref37) {
    var contractAddress, objectId, versionHash;
    return _regeneratorRuntime.wrap(function _callee18$(_context18) {
      while (1) switch (_context18.prev = _context18.next) {
        case 0:
          contractAddress = _ref37.contractAddress, objectId = _ref37.objectId, versionHash = _ref37.versionHash;
          if (!contractAddress) {
            _context18.next = 6;
            break;
          }
          ValidateAddress(contractAddress);
          objectId = Utils.AddressToObjectId(contractAddress);
          _context18.next = 18;
          break;
        case 6:
          if (!versionHash) {
            _context18.next = 12;
            break;
          }
          ValidateVersion(versionHash);
          objectId = this.utils.DecodeVersionHash(versionHash).objectId;
          contractAddress = Utils.HashToAddress(objectId);
          _context18.next = 18;
          break;
        case 12:
          if (!objectId) {
            _context18.next = 17;
            break;
          }
          ValidateObject(objectId);
          contractAddress = Utils.HashToAddress(objectId);
          _context18.next = 18;
          break;
        case 17:
          throw Error("contractAddress or objectId or versionHash not specified");
        case 18:
          return _context18.abrupt("return", {
            contractAddress: contractAddress,
            objectId: objectId
          });
        case 19:
        case "end":
          return _context18.stop();
      }
    }, _callee18, this);
  }));
  return function GetObjectIDAndContractAddress(_x16) {
    return _ref38.apply(this, arguments);
  };
}();

/**
 * Retrieve the ID of the tenant admin group set for the specified object
 *
 * @methodGroup Tenant
 * @namedParams
 * @param {string=} contractAddress - The address of the object
 * @param {string=} objectId - The ID of the object
 * @param {string=} versionHash - A version hash of the object
 *
 * @returns {Promise<string|undefined>}
 */
exports.TenantId = /*#__PURE__*/function () {
  var _ref40 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee19(_ref39) {
    var contractAddress, objectId, versionHash, tenantId, hasGetMetaMethod, libraryId;
    return _regeneratorRuntime.wrap(function _callee19$(_context19) {
      while (1) switch (_context19.prev = _context19.next) {
        case 0:
          contractAddress = _ref39.contractAddress, objectId = _ref39.objectId, versionHash = _ref39.versionHash;
          _context19.next = 3;
          return GetObjectIDAndContractAddress({
            contractAddress: contractAddress,
            objectId: objectId,
            versionHash: versionHash
          });
        case 3:
          objectInfo = _context19.sent;
          contractAddress = objectInfo.contractAddress;
          objectId = objectInfo.objectId;
          _context19.prev = 6;
          _context19.next = 9;
          return this.authClient.ContractHasMethod({
            contractAddress: contractAddress,
            methodName: "getMeta"
          });
        case 9:
          hasGetMetaMethod = _context19.sent;
          if (!hasGetMetaMethod) {
            _context19.next = 14;
            break;
          }
          _context19.next = 13;
          return this.ContractMetadata({
            contractAddress: contractAddress,
            metadataKey: "_tenantId"
          });
        case 13:
          tenantId = _context19.sent;
        case 14:
          if (!(tenantId === undefined)) {
            _context19.next = 21;
            break;
          }
          _context19.next = 17;
          return this.ContentObjectLibraryId({
            objectId: objectId
          });
        case 17:
          libraryId = _context19.sent;
          _context19.next = 20;
          return this.ContentObjectMetadata({
            libraryId: libraryId,
            objectId: objectId,
            metadataSubtree: "tenantId"
          });
        case 20:
          tenantId = _context19.sent;
        case 21:
          return _context19.abrupt("return", tenantId);
        case 24:
          _context19.prev = 24;
          _context19.t0 = _context19["catch"](6);
          return _context19.abrupt("return", "");
        case 27:
        case "end":
          return _context19.stop();
      }
    }, _callee19, this, [[6, 24]]);
  }));
  return function (_x17) {
    return _ref40.apply(this, arguments);
  };
}();

/**
 * Retrieve the ID of the tenant contract for the specified object
 *
 * @methodGroup Tenant
 * @namedParams
 * @param {string=} contractAddress - The address of the object
 * @param {string=} objectId - The ID of the object
 * @param {string=} versionHash - A version hash of the object
 *
 * @returns {Promise<string|undefined>}
 */
exports.TenantContractId = /*#__PURE__*/function () {
  var _ref42 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee20(_ref41) {
    var contractAddress, objectId, versionHash, hasGetMetaMethod, _tenantContractId, libraryId;
    return _regeneratorRuntime.wrap(function _callee20$(_context20) {
      while (1) switch (_context20.prev = _context20.next) {
        case 0:
          contractAddress = _ref41.contractAddress, objectId = _ref41.objectId, versionHash = _ref41.versionHash;
          _context20.next = 3;
          return GetObjectIDAndContractAddress({
            contractAddress: contractAddress,
            objectId: objectId,
            versionHash: versionHash
          });
        case 3:
          objectInfo = _context20.sent;
          contractAddress = objectInfo.contractAddress;
          objectId = objectInfo.objectId;
          _context20.prev = 6;
          _context20.next = 9;
          return this.authClient.ContractHasMethod({
            contractAddress: contractAddress,
            methodName: "getMeta"
          });
        case 9:
          hasGetMetaMethod = _context20.sent;
          if (!hasGetMetaMethod) {
            _context20.next = 14;
            break;
          }
          _context20.next = 13;
          return this.ContractMetadata({
            contractAddress: contractAddress,
            metadataKey: "_ELV_TENANT_ID"
          });
        case 13:
          _tenantContractId = _context20.sent;
        case 14:
          if (!(_tenantContractId === undefined)) {
            _context20.next = 21;
            break;
          }
          _context20.next = 17;
          return this.ContentObjectLibraryId({
            objectId: objectId
          });
        case 17:
          libraryId = _context20.sent;
          _context20.next = 20;
          return this.ContentObjectMetadata({
            libraryId: libraryId,
            objectId: objectId,
            metadataSubtree: "tenantContractId"
          });
        case 20:
          _tenantContractId = _context20.sent;
        case 21:
          return _context20.abrupt("return", _tenantContractId);
        case 24:
          _context20.prev = 24;
          _context20.t0 = _context20["catch"](6);
          return _context20.abrupt("return", "");
        case 27:
        case "end":
          return _context20.stop();
      }
    }, _callee20, this, [[6, 24]]);
  }));
  return function (_x18) {
    return _ref42.apply(this, arguments);
  };
}();

/**
 * Set the tenant contract ID and tenant admin group ID for the specified object
 * when tenant admin group ID is provided
 *
 * @methodGroup Tenant
 * @namedParams
 * @param {string=} contractAddress - The address of the object
 * @param {string=} objectId - The ID of the object
 * @param {string=} versionHash - A version hash of the object
 * @param {string} tenantContractId - The tenant contract ID to set
 * @param {string} tenantId - The tenant ID to set
 *
 * @returns {Promise<{tenantId: (undefined|string), tenantContractId}>}
 */
exports.SetTenantId = /*#__PURE__*/function () {
  var _ref44 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee21(_ref43) {
    var contractAddress, objectId, versionHash, tenantId, objectVersion, version;
    return _regeneratorRuntime.wrap(function _callee21$(_context21) {
      while (1) switch (_context21.prev = _context21.next) {
        case 0:
          contractAddress = _ref43.contractAddress, objectId = _ref43.objectId, versionHash = _ref43.versionHash, tenantId = _ref43.tenantId;
          _context21.next = 3;
          return GetObjectIDAndContractAddress({
            contractAddress: contractAddress,
            objectId: objectId,
            versionHash: versionHash
          });
        case 3:
          objectInfo = _context21.sent;
          contractAddress = objectInfo.contractAddress;
          objectId = objectInfo.objectId;
          _context21.next = 8;
          return this.authClient.AccessType(objectId);
        case 8:
          objectVersion = _context21.sent;
          if (!(objectVersion !== this.authClient.ACCESS_TYPES.GROUP && objectVersion !== this.authClient.ACCESS_TYPES.WALLET && objectVersion !== this.authClient.ACCESS_TYPES.LIBRARY && objectVersion !== this.authClient.ACCESS_TYPES.TYPE && objectVersion !== this.authClient.ACCESS_TYPES.TENANT)) {
            _context21.next = 11;
            break;
          }
          throw Error("Invalid object ID: ".concat(objectId, ", \n    applicable only for wallet,group, library or content_type object."));
        case 11:
          ValidateObject(tenantId);
          if (!(!tenantId.startsWith("iten") || !Utils.ValidHash(tenantId))) {
            _context21.next = 14;
            break;
          }
          throw Error("Invalid tenant ID: ".concat(tenantId));
        case 14:
          _context21.next = 16;
          return this.authClient.AccessType(tenantId);
        case 16:
          version = _context21.sent;
          if (!(version !== this.authClient.ACCESS_TYPES.GROUP)) {
            _context21.next = 19;
            break;
          }
          throw Error("Invalid tenant ID: " + tenantId);
        case 19:
          _context21.next = 21;
          return this.TenantContractId({
            objectId: tenantId
          });
        case 21:
          tenantContractId = _context21.sent;
          if (!tenantContractId) {
            _context21.next = 28;
            break;
          }
          _context21.next = 25;
          return this.SetTenantContractId({
            contractAddress: contractAddress,
            objectId: objectId,
            versionHash: versionHash,
            tenantContractId: tenantContractId
          });
        case 25:
          return _context21.abrupt("return", _context21.sent);
        case 28:
          throw Error("Invalid tenantId: tenant contract id not found");
        case 29:
        case "end":
          return _context21.stop();
      }
    }, _callee21, this);
  }));
  return function (_x19) {
    return _ref44.apply(this, arguments);
  };
}();

/**
 * Set the tenant contract ID and tenant admin group ID for the specified object
 * when tenant contract ID is provided
 *
 * @methodGroup Tenant
 * @namedParams
 * @param {string=} contractAddress - The address of the object
 * @param {string=} objectId - The ID of the object
 * @param {string=} versionHash - A version hash of the object
 * @param {string} tenantContractId - The tenant contract ID to set
 *
 * @returns {Promise<{tenantId: (undefined|string), tenantContractId}>}
 */
exports.SetTenantContractId = /*#__PURE__*/function () {
  var _ref46 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee22(_ref45) {
    var contractAddress, objectId, versionHash, tenantContractId, objectVersion, tenantAddress, version, tenantAdminGroupAddress, hasPutMetaMethod, libraryId, editRequest;
    return _regeneratorRuntime.wrap(function _callee22$(_context22) {
      while (1) switch (_context22.prev = _context22.next) {
        case 0:
          contractAddress = _ref45.contractAddress, objectId = _ref45.objectId, versionHash = _ref45.versionHash, tenantContractId = _ref45.tenantContractId;
          _context22.next = 3;
          return GetObjectIDAndContractAddress({
            contractAddress: contractAddress,
            objectId: objectId,
            versionHash: versionHash
          });
        case 3:
          objectInfo = _context22.sent;
          contractAddress = objectInfo.contractAddress;
          objectId = objectInfo.objectId;
          _context22.next = 8;
          return this.authClient.AccessType(objectId);
        case 8:
          objectVersion = _context22.sent;
          if (!(objectVersion !== this.authClient.ACCESS_TYPES.GROUP && objectVersion !== this.authClient.ACCESS_TYPES.WALLET && objectVersion !== this.authClient.ACCESS_TYPES.LIBRARY && objectVersion !== this.authClient.ACCESS_TYPES.TYPE && objectVersion !== this.authClient.ACCESS_TYPES.TENANT)) {
            _context22.next = 11;
            break;
          }
          throw Error("Invalid object ID: ".concat(objectId, ", \n    applicable only for wallet,group, library or content_type object."));
        case 11:
          ValidateObject(tenantContractId);
          if (!(tenantContractId && (!tenantContractId.startsWith("iten") || !Utils.ValidHash(tenantContractId)))) {
            _context22.next = 14;
            break;
          }
          throw Error("Invalid tenant ID: ".concat(tenantContractId));
        case 14:
          tenantAddress = Utils.HashToAddress(tenantContractId);
          _context22.next = 17;
          return this.authClient.AccessType(tenantContractId);
        case 17:
          version = _context22.sent;
          if (!(version !== this.authClient.ACCESS_TYPES.TENANT)) {
            _context22.next = 20;
            break;
          }
          throw Error("Invalid tenant ID: " + tenantContractId);
        case 20:
          _context22.next = 22;
          return this.CallContractMethod({
            contractAddress: tenantAddress,
            methodName: "groupsMapping",
            methodArgs: ["tenant_admin", 0],
            formatArguments: true
          });
        case 22:
          tenantAdminGroupAddress = _context22.sent;
          _context22.next = 25;
          return this.authClient.ContractHasMethod({
            contractAddress: contractAddress,
            methodName: "putMeta"
          });
        case 25:
          hasPutMetaMethod = _context22.sent;
          if (!hasPutMetaMethod) {
            _context22.next = 37;
            break;
          }
          _context22.next = 29;
          return this.ReplaceContractMetadata({
            contractAddress: contractAddress,
            metadataKey: "_ELV_TENANT_ID",
            metadata: tenantContractId
          });
        case 29:
          if (!tenantAdminGroupAddress) {
            _context22.next = 34;
            break;
          }
          _context22.next = 32;
          return this.ReplaceContractMetadata({
            contractAddress: contractAddress,
            metadataKey: "_tenantId",
            metadata: "iten".concat(Utils.AddressToHash(tenantAdminGroupAddress))
          });
        case 32:
          _context22.next = 35;
          break;
        case 34:
          // eslint-disable-next-line no-console
          console.warn("No tenant ID associated with current tenant.");
        case 35:
          _context22.next = 47;
          break;
        case 37:
          _context22.next = 39;
          return this.ContentObjectLibraryId({
            objectId: objectId
          });
        case 39:
          libraryId = _context22.sent;
          _context22.next = 42;
          return this.EditContentObject({
            libraryId: libraryId,
            objectId: objectId
          });
        case 42:
          editRequest = _context22.sent;
          _context22.next = 45;
          return this.MergeMetadata({
            libraryId: libraryId,
            objectId: objectId,
            writeToken: editRequest.write_token,
            metadata: {
              tenantContractId: tenantContractId,
              tenantId: !tenantAdminGroupAddress ? undefined : "iten".concat(Utils.AddressToHash(tenantAdminGroupAddress))
            }
          });
        case 45:
          _context22.next = 47;
          return this.FinalizeContentObject({
            libraryId: libraryId,
            objectId: objectId,
            writeToken: editRequest.write_token,
            commitMessage: "set tenant_contract_id"
          });
        case 47:
          return _context22.abrupt("return", {
            tenantContractId: tenantContractId,
            tenantId: !tenantAdminGroupAddress ? undefined : "iten".concat(Utils.AddressToHash(tenantAdminGroupAddress))
          });
        case 48:
        case "end":
          return _context22.stop();
      }
    }, _callee22, this);
  }));
  return function (_x20) {
    return _ref46.apply(this, arguments);
  };
}();

/**
 * Remove the tenant contract ID and tenant admin group ID for the specified object
 *
 * @methodGroup Tenant
 * @namedParams
 * @param {string=} contractAddress - The address of the object
 * @param {string=} objectId - The ID of the object
 * @param {string=} versionHash - A version hash of the object
 *
 * @returns {Promise<void>}
 */
exports.ResetTenantId = /*#__PURE__*/function () {
  var _ref48 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee23(_ref47) {
    var contractAddress, objectId, versionHash, objectVersion, tenantContractId, tenantId, hasPutMetaMethod, libraryId, editRequest;
    return _regeneratorRuntime.wrap(function _callee23$(_context23) {
      while (1) switch (_context23.prev = _context23.next) {
        case 0:
          contractAddress = _ref47.contractAddress, objectId = _ref47.objectId, versionHash = _ref47.versionHash;
          _context23.next = 3;
          return GetObjectIDAndContractAddress({
            contractAddress: contractAddress,
            objectId: objectId,
            versionHash: versionHash
          });
        case 3:
          objectInfo = _context23.sent;
          contractAddress = objectInfo.contractAddress;
          objectId = objectInfo.objectId;
          _context23.next = 8;
          return this.authClient.AccessType(objectId);
        case 8:
          objectVersion = _context23.sent;
          if (!(objectVersion !== this.authClient.ACCESS_TYPES.GROUP && objectVersion !== this.authClient.ACCESS_TYPES.WALLET && objectVersion !== this.authClient.ACCESS_TYPES.LIBRARY && objectVersion !== this.authClient.ACCESS_TYPES.TYPE && objectVersion !== this.authClient.ACCESS_TYPES.TENANT)) {
            _context23.next = 11;
            break;
          }
          throw Error("Invalid object ID: ".concat(objectId, ", \n    applicable only for wallet,group, library or content_type object."));
        case 11:
          tenantContractId = this.TenantContractId({
            objectId: objectId
          });
          tenantId = this.TenantId({
            objectId: objectId
          });
          if (!(tenantContractId || tenantId)) {
            _context23.next = 36;
            break;
          }
          _context23.next = 16;
          return this.authClient.ContractHasMethod({
            contractAddress: contractAddress,
            methodName: "putMeta"
          });
        case 16:
          hasPutMetaMethod = _context23.sent;
          if (!hasPutMetaMethod) {
            _context23.next = 24;
            break;
          }
          _context23.next = 20;
          return this.ReplaceContractMetadata({
            contractAddress: contractAddress,
            metadataKey: "_ELV_TENANT_ID",
            metadata: ""
          });
        case 20:
          _context23.next = 22;
          return this.ReplaceContractMetadata({
            contractAddress: contractAddress,
            metadataKey: "_tenantId",
            metadata: ""
          });
        case 22:
          _context23.next = 34;
          break;
        case 24:
          _context23.next = 26;
          return this.ContentObjectLibraryId({
            objectId: objectId
          });
        case 26:
          libraryId = _context23.sent;
          _context23.next = 29;
          return this.EditContentObject({
            libraryId: libraryId,
            objectId: objectId
          });
        case 29:
          editRequest = _context23.sent;
          _context23.next = 32;
          return this.MergeMetadata({
            libraryId: libraryId,
            objectId: objectId,
            writeToken: editRequest.write_token,
            metadata: {
              tenantContractId: undefined,
              tenantId: undefined
            }
          });
        case 32:
          _context23.next = 34;
          return this.FinalizeContentObject({
            libraryId: libraryId,
            objectId: objectId,
            writeToken: editRequest.write_token,
            commitMessage: "remove tenant_contract_id"
          });
        case 34:
          _context23.next = 37;
          break;
        case 36:
          // eslint-disable-next-line no-console
          console.warn("No tenant ID associated with current tenant.");
        case 37:
        case "end":
          return _context23.stop();
      }
    }, _callee23, this);
  }));
  return function (_x21) {
    return _ref48.apply(this, arguments);
  };
}();