var _defineProperty = require("@babel/runtime/helpers/defineProperty");
var _regeneratorRuntime = require("@babel/runtime/regenerator");
var _asyncToGenerator = require("@babel/runtime/helpers/asyncToGenerator");
function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
/**
 * Methods for creating and managing shared authorization tokens
 *
 * @module ElvClient/Shares
 */

var UrlJoin = require("url-join");

/**
 * Create a share
 *
 * @namedParams
 * @param {string} objectId - The object to create a share for
 * @param {Date} expiresAt - The expiration time of the share
 * @param {Object=} params - Additional parameters
 *
 * @returns {Promise<Object>} - Info about the created share
 */
exports.CreateShare = /*#__PURE__*/function () {
  var _ref2 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(_ref) {
    var objectId, expiresAt, _ref$params, params, tenantId, token, share;
    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) switch (_context.prev = _context.next) {
        case 0:
          objectId = _ref.objectId, expiresAt = _ref.expiresAt, _ref$params = _ref.params, params = _ref$params === void 0 ? {} : _ref$params;
          _context.next = 3;
          return this.userProfileClient.TenantContractId();
        case 3:
          tenantId = _context.sent;
          _context.next = 6;
          return this.CreateFabricToken({});
        case 6:
          token = _context.sent;
          params.object_id = objectId;
          if (expiresAt) {
            params.end_time = Math.floor(new Date(expiresAt).getTime() / 1000);
          }
          _context.next = 11;
          return this.MakeAuthServiceRequest({
            path: UrlJoin("as", "sharing", tenantId, "share"),
            method: "POST",
            format: "JSON",
            body: params,
            headers: {
              Authorization: "Bearer ".concat(token)
            }
          });
        case 11:
          share = _context.sent;
          share.start_time = share.start_time ? new Date(share.start_time * 1000).toISOString() : undefined;
          share.end_time = share.end_time ? new Date(share.end_time * 1000).toISOString() : undefined;
          return _context.abrupt("return", share);
        case 15:
        case "end":
          return _context.stop();
      }
    }, _callee, this);
  }));
  return function (_x) {
    return _ref2.apply(this, arguments);
  };
}();

/**
 * List shares associated with your tenancy
 *
 * @namedParams
 * @param {string=} objectId - If specified, the results will be limited shares for the specified object
 * @param {number=} limit=100 - Maximum number of results to return
 * @param {number=} offset=0 - Offset from which to return results
 * @param {Object=} params={} - Additional parameters
 *
 * @returns {Promise<Array<Object>>} - Info about the shares
 */
exports.Shares = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2() {
  var _ref4,
    objectId,
    _ref4$limit,
    limit,
    _ref4$offset,
    offset,
    _ref4$params,
    params,
    tenantId,
    token,
    response,
    _args2 = arguments;
  return _regeneratorRuntime.wrap(function _callee2$(_context2) {
    while (1) switch (_context2.prev = _context2.next) {
      case 0:
        _ref4 = _args2.length > 0 && _args2[0] !== undefined ? _args2[0] : {}, objectId = _ref4.objectId, _ref4$limit = _ref4.limit, limit = _ref4$limit === void 0 ? 100 : _ref4$limit, _ref4$offset = _ref4.offset, offset = _ref4$offset === void 0 ? 0 : _ref4$offset, _ref4$params = _ref4.params, params = _ref4$params === void 0 ? {} : _ref4$params;
        _context2.next = 3;
        return this.userProfileClient.TenantContractId();
      case 3:
        tenantId = _context2.sent;
        _context2.next = 6;
        return this.CreateFabricToken({});
      case 6:
        token = _context2.sent;
        _context2.next = 9;
        return this.MakeAuthServiceRequest({
          path: UrlJoin("as", "sharing", tenantId, "shares"),
          method: objectId || Object.keys(params).length > 0 ? "POST" : "GET",
          queryParams: {
            limit: limit,
            offset: offset
          },
          body: objectId ? _objectSpread({
            object_id: objectId
          }, params) : undefined,
          format: "JSON",
          headers: {
            Authorization: "Bearer ".concat(token)
          }
        });
      case 9:
        response = _context2.sent;
        // Convert dates from seconds to date strings
        if (response && response.shares) {
          response.shares = response.shares.map(function (share) {
            return _objectSpread(_objectSpread({}, share), {}, {
              start_time: share.start_time ? new Date(share.start_time * 1000).toISOString() : null,
              end_time: share.end_time ? new Date(share.end_time * 1000).toISOString() : null
            });
          });
        }
        return _context2.abrupt("return", response);
      case 12:
      case "end":
        return _context2.stop();
    }
  }, _callee2, this);
}));

/**
 * Update the specified share
 *
 * @namedParams
 * @param {string} shareId - The ID of the share to modify
 * @param {Date=} expiresAt - The new expiration time of the share
 * @param {Object=} params - Additional parameters
 *
 * @returns {Promise<Object>} - Info about the updated share
 */
exports.UpdateShare = /*#__PURE__*/function () {
  var _ref6 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee3(_ref5) {
    var shareId, expiresAt, _ref5$params, params, tenantId, token;
    return _regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) switch (_context3.prev = _context3.next) {
        case 0:
          shareId = _ref5.shareId, expiresAt = _ref5.expiresAt, _ref5$params = _ref5.params, params = _ref5$params === void 0 ? {} : _ref5$params;
          _context3.next = 3;
          return this.userProfileClient.TenantContractId();
        case 3:
          tenantId = _context3.sent;
          _context3.next = 6;
          return this.CreateFabricToken({});
        case 6:
          token = _context3.sent;
          if (expiresAt) {
            params.end_time = Math.floor(new Date(expiresAt).getTime() / 1000);
          }
          _context3.next = 10;
          return this.MakeAuthServiceRequest({
            path: UrlJoin("as", "sharing", tenantId, "share", shareId),
            method: "PUT",
            format: "JSON",
            body: params,
            headers: {
              Authorization: "Bearer ".concat(token)
            }
          });
        case 10:
          return _context3.abrupt("return", _context3.sent);
        case 11:
        case "end":
          return _context3.stop();
      }
    }, _callee3, this);
  }));
  return function (_x2) {
    return _ref6.apply(this, arguments);
  };
}();

/**
 * Revoke the specified share
 *
 * @namedParams
 * @param {string} shareId - The ID of the share to modify
 */
exports.RevokeShare = /*#__PURE__*/function () {
  var _ref8 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee4(_ref7) {
    var shareId, tenantId, token;
    return _regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) switch (_context4.prev = _context4.next) {
        case 0:
          shareId = _ref7.shareId;
          _context4.next = 3;
          return this.userProfileClient.TenantContractId();
        case 3:
          tenantId = _context4.sent;
          _context4.next = 6;
          return this.CreateFabricToken({});
        case 6:
          token = _context4.sent;
          _context4.next = 9;
          return this.MakeAuthServiceRequest({
            path: UrlJoin("as", "sharing", tenantId, "share", shareId, "revoke"),
            method: "PUT",
            format: "JSON",
            headers: {
              Authorization: "Bearer ".concat(token)
            }
          });
        case 9:
          return _context4.abrupt("return", _context4.sent);
        case 10:
        case "end":
          return _context4.stop();
      }
    }, _callee4, this);
  }));
  return function (_x3) {
    return _ref8.apply(this, arguments);
  };
}();

/**
 * Retrieve info about the specified share
 *
 * @namedParams
 * @param {string} shareId - The ID of the share
 *
 * @returns {Promise<Object>} - Info about the share
 */
exports.ShareInfo = /*#__PURE__*/function () {
  var _ref10 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee5(_ref9) {
    var shareId, _yield$this$MakeAuthS, share;
    return _regeneratorRuntime.wrap(function _callee5$(_context5) {
      while (1) switch (_context5.prev = _context5.next) {
        case 0:
          shareId = _ref9.shareId;
          _context5.next = 3;
          return this.MakeAuthServiceRequest({
            path: UrlJoin("as", "sharing", "share", shareId, "info"),
            method: "GET",
            format: "JSON"
          });
        case 3:
          _yield$this$MakeAuthS = _context5.sent;
          share = _yield$this$MakeAuthS.share;
          if (share.start_time) {
            share.start_time = new Date(share.start_time * 1000).toISOString();
          }
          if (share.end_time) {
            share.end_time = new Date(share.end_time * 1000).toISOString();
          }
          return _context5.abrupt("return", share);
        case 8:
        case "end":
          return _context5.stop();
      }
    }, _callee5, this);
  }));
  return function (_x4) {
    return _ref10.apply(this, arguments);
  };
}();

/**
 * Retrieve the authorization token for the specified share
 *
 * @namedParams
 * @param {string} shareId - The ID of the share
 *
 * @returns {Promise<Object>} - The authorization token for the share
 */
exports.RedeemShareToken = /*#__PURE__*/function () {
  var _ref12 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee6(_ref11) {
    var shareId;
    return _regeneratorRuntime.wrap(function _callee6$(_context6) {
      while (1) switch (_context6.prev = _context6.next) {
        case 0:
          shareId = _ref11.shareId;
          _context6.next = 3;
          return this.MakeAuthServiceRequest({
            path: UrlJoin("as", "sharing", "share", shareId, "token"),
            method: "GET",
            format: "JSON"
          });
        case 3:
          return _context6.abrupt("return", _context6.sent.token);
        case 4:
        case "end":
          return _context6.stop();
      }
    }, _callee6, this);
  }));
  return function (_x5) {
    return _ref12.apply(this, arguments);
  };
}();