var _defineProperty = require("@babel/runtime/helpers/defineProperty");
var _regeneratorRuntime = require("@babel/runtime/regenerator");
var _asyncToGenerator = require("@babel/runtime/helpers/asyncToGenerator");
function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
var Utils = require("../Utils");
var UrlJoin = require("url-join");
var _require = require("./Utils"),
  FormatNFTDetails = _require.FormatNFTDetails,
  FormatNFTMetadata = _require.FormatNFTMetadata,
  FormatNFT = _require.FormatNFT;
var MergeWith = require("lodash/mergeWith");

/**
 * Methods
 *
 * @module ClientMethods
 */

/* USER INFO */

/**
 * <b><i>Requires login</i></b>
 *
 * Retrieve information about the user, including the address, wallet type, and (for custodial users) email address.
 *
 * @methodGroup User
 *
 * @returns {Object} - User info
 */
exports.UserInfo = function () {
  if (!this.loggedIn) {
    return;
  }
  return {
    name: this.__authorization.email || this.UserAddress(),
    address: this.UserAddress(),
    email: this.__authorization.email,
    walletType: this.__authorization.walletType,
    walletName: this.__authorization.walletName
  };
};

/**
 * <b><i>Requires login</i></b>
 *
 * Retrieve the address of the current user.
 *
 * @methodGroup User
 *
 * @returns {string} - The address of the current user
 */
exports.UserAddress = function () {
  if (!this.loggedIn) {
    return;
  }
  return this.client.utils.DecodeSignedToken(this.AuthToken()).payload.adr;
};

/**
 * <b><i>Requires login</i></b>
 *
 * Retrieve the fund balances for the current user
 *
 * @methodGroup User
 * @returns {Promise<Object>} - Returns balances for the user. All values are in USD.
 *  <ul>
 *  <li>- totalWalletBalance - Total balance of the users sales and wallet balance purchases</li>
 *  <li>- availableWalletBalance - Balance available for purchasing items</li>
 *  <li>- pendingWalletBalance - Balance unavailable for purchasing items</li>
 *  <li>- withdrawableWalletBalance - Amount that is available for withdrawal</li>
 *  <li>- usedBalance - <i>(Only included if user has set up Solana link with the Phantom wallet)</i> Available USDC balance of the user's Solana wallet</li>
 *  </ul>
 */
exports.UserWalletBalance = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
  var checkOnboard,
    _yield$this$client$ut,
    balance,
    usage_hold,
    payout_hold,
    locked_offer_balance,
    stripe_id,
    stripe_payouts_enabled,
    userStripeId,
    userStripeEnabled,
    totalWalletBalance,
    lockedWalletBalance,
    availableWalletBalance,
    pendingWalletBalance,
    withdrawableWalletBalance,
    rootUrl,
    balances,
    _args = arguments;
  return _regeneratorRuntime.wrap(function _callee$(_context) {
    while (1) switch (_context.prev = _context.next) {
      case 0:
        checkOnboard = _args.length > 0 && _args[0] !== undefined ? _args[0] : false;
        if (this.loggedIn) {
          _context.next = 3;
          break;
        }
        return _context.abrupt("return");
      case 3:
        _context.t0 = this.client.utils;
        _context.next = 6;
        return this.client.authClient.MakeAuthServiceRequest({
          path: UrlJoin("as", "wlt", "mkt", "bal"),
          method: "GET",
          headers: {
            Authorization: "Bearer ".concat(this.AuthToken())
          }
        });
      case 6:
        _context.t1 = _context.sent;
        _context.next = 9;
        return _context.t0.ResponseToJson.call(_context.t0, _context.t1);
      case 9:
        _yield$this$client$ut = _context.sent;
        balance = _yield$this$client$ut.balance;
        usage_hold = _yield$this$client$ut.usage_hold;
        payout_hold = _yield$this$client$ut.payout_hold;
        locked_offer_balance = _yield$this$client$ut.locked_offer_balance;
        stripe_id = _yield$this$client$ut.stripe_id;
        stripe_payouts_enabled = _yield$this$client$ut.stripe_payouts_enabled;
        userStripeId = stripe_id;
        userStripeEnabled = stripe_payouts_enabled;
        totalWalletBalance = parseFloat(balance || 0);
        lockedWalletBalance = parseFloat(locked_offer_balance || 0);
        availableWalletBalance = Math.max(0, totalWalletBalance - parseFloat(usage_hold || 0) - lockedWalletBalance);
        pendingWalletBalance = Math.max(0, totalWalletBalance - availableWalletBalance);
        withdrawableWalletBalance = Math.max(0, totalWalletBalance - parseFloat(Math.max(payout_hold, lockedWalletBalance) || 0));
        if (!(checkOnboard && stripe_id && !stripe_payouts_enabled)) {
          _context.next = 30;
          break;
        }
        // Refresh stripe enabled flag
        rootUrl = new URL(UrlJoin(window.location.origin, window.location.pathname)).toString();
        _context.next = 27;
        return this.client.authClient.MakeAuthServiceRequest({
          path: UrlJoin("as", "wlt", "onb", "stripe"),
          method: "POST",
          body: {
            country: "US",
            mode: this.mode,
            refresh_url: rootUrl.toString(),
            return_url: rootUrl.toString()
          },
          headers: {
            Authorization: "Bearer ".concat(this.AuthToken())
          }
        });
      case 27:
        _context.next = 29;
        return this.UserWalletBalance(false);
      case 29:
        return _context.abrupt("return", _context.sent);
      case 30:
        balances = {
          totalWalletBalance: totalWalletBalance,
          availableWalletBalance: availableWalletBalance,
          lockedWalletBalance: lockedWalletBalance,
          pendingWalletBalance: pendingWalletBalance,
          withdrawableWalletBalance: withdrawableWalletBalance
        };
        if (userStripeEnabled) {
          balances.userStripeId = userStripeId;
          balances.userStripeEnabled = userStripeEnabled;
        }

        // TODO: integrate
        /*
        if(cryptoStore.usdcConnected) {
          balances.usdcBalance = cryptoStore.phantomUSDCBalance;
        }
          */
        return _context.abrupt("return", balances);
      case 33:
      case "end":
        return _context.stop();
    }
  }, _callee, this);
}));

/**
 * Retrieve all valid names for filtering user items. Full item names are required for filtering results by name.
 *
 * Specify marketplace information to filter the results to only items offered in that marketplace.
 *
 * @methodGroup User
 * @namedParams
 * @param {string=} userAddress - Address of a user
 * @param {Object=} marketplaceParams - Parameters of a marketplace to filter results by
 *
 * @returns {Promise<Array<String>>} - A list of item names
 */
exports.UserItemNames = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2() {
  var _ref3,
    marketplaceParams,
    userAddress,
    filters,
    _args2 = arguments;
  return _regeneratorRuntime.wrap(function _callee2$(_context2) {
    while (1) switch (_context2.prev = _context2.next) {
      case 0:
        _ref3 = _args2.length > 0 && _args2[0] !== undefined ? _args2[0] : {}, marketplaceParams = _ref3.marketplaceParams, userAddress = _ref3.userAddress;
        filters = [];
        if (!marketplaceParams) {
          _context2.next = 10;
          break;
        }
        _context2.t0 = filters;
        _context2.t1 = "tenant:eq:";
        _context2.next = 7;
        return this.MarketplaceInfo({
          marketplaceParams: marketplaceParams
        });
      case 7:
        _context2.t2 = _context2.sent.tenantId;
        _context2.t3 = _context2.t1.concat.call(_context2.t1, _context2.t2);
        _context2.t0.push.call(_context2.t0, _context2.t3);
      case 10:
        if (userAddress) {
          filters.push("wlt:eq:".concat(Utils.FormatAddress(userAddress)));
        }
        _context2.t4 = Utils;
        _context2.next = 14;
        return this.client.authClient.MakeAuthServiceRequest({
          path: UrlJoin("as", "wlt", "names"),
          method: "GET",
          queryParams: {
            filter: filters
          }
        });
      case 14:
        _context2.t5 = _context2.sent;
        _context2.next = 17;
        return _context2.t4.ResponseToJson.call(_context2.t4, _context2.t5);
      case 17:
        return _context2.abrupt("return", _context2.sent);
      case 18:
      case "end":
        return _context2.stop();
    }
  }, _callee2, this);
}));

/**
 * Retrieve all valid edition names for filtering the specified item. Full edition names are required for filtering results by edition.
 *
 * Specify marketplace information to filter the results to only items offered in that marketplace.
 *
 * @methodGroup User
 * @namedParams
 * @param {string} displayName - Name of an item
 *
 * @returns {Promise<Array<String>>} - A list of item editions
 */
exports.UserItemEditionNames = /*#__PURE__*/function () {
  var _ref5 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee3(_ref4) {
    var displayName;
    return _regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) switch (_context3.prev = _context3.next) {
        case 0:
          displayName = _ref4.displayName;
          _context3.t0 = Utils;
          _context3.next = 4;
          return this.client.authClient.MakeAuthServiceRequest({
            path: UrlJoin("as", "wlt", "editions"),
            method: "GET",
            queryParams: {
              filter: "meta/display_name:eq:".concat(displayName)
            }
          });
        case 4:
          _context3.t1 = _context3.sent;
          _context3.next = 7;
          return _context3.t0.ResponseToJson.call(_context3.t0, _context3.t1);
        case 7:
          return _context3.abrupt("return", _context3.sent);
        case 8:
        case "end":
          return _context3.stop();
      }
    }, _callee3, this);
  }));
  return function (_x) {
    return _ref5.apply(this, arguments);
  };
}();

/**
 * Retrieve all valid attribute names and values. Full attribute names and values are required for filtering results by attribute.
 *
 * Specify marketplace information to filter the results to only items offered in that marketplace.
 *
 * @methodGroup User
 * @namedParams
 * @param {string=} userAddress - Address of a user
 * @param {string=} displayName - Name of an item
 * @param {Object=} marketplaceParams - Parameters of a marketplace to filter results by
 *
 * @returns {Promise<Array<String>>} - A list of item names
 */
exports.UserItemAttributes = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee4() {
  var _ref7,
    marketplaceParams,
    displayName,
    userAddress,
    filters,
    attributes,
    _args4 = arguments;
  return _regeneratorRuntime.wrap(function _callee4$(_context4) {
    while (1) switch (_context4.prev = _context4.next) {
      case 0:
        _ref7 = _args4.length > 0 && _args4[0] !== undefined ? _args4[0] : {}, marketplaceParams = _ref7.marketplaceParams, displayName = _ref7.displayName, userAddress = _ref7.userAddress;
        filters = [];
        if (!marketplaceParams) {
          _context4.next = 10;
          break;
        }
        _context4.t0 = filters;
        _context4.t1 = "tenant:eq:";
        _context4.next = 7;
        return this.MarketplaceInfo({
          marketplaceParams: marketplaceParams
        });
      case 7:
        _context4.t2 = _context4.sent.tenantId;
        _context4.t3 = _context4.t1.concat.call(_context4.t1, _context4.t2);
        _context4.t0.push.call(_context4.t0, _context4.t3);
      case 10:
        if (userAddress) {
          filters.push("wlt:eq:".concat(Utils.FormatAddress(userAddress)));
        }
        if (displayName) {
          filters.push("meta/display_name:eq:".concat(displayName));
        }
        _context4.t4 = Utils;
        _context4.next = 15;
        return this.client.authClient.MakeAuthServiceRequest({
          path: UrlJoin("as", "wlt", "attributes"),
          method: "GET",
          queryParams: {
            filter: filters
          }
        });
      case 15:
        _context4.t5 = _context4.sent;
        _context4.next = 18;
        return _context4.t4.ResponseToJson.call(_context4.t4, _context4.t5);
      case 18:
        attributes = _context4.sent;
        return _context4.abrupt("return", attributes.map(function (_ref8) {
          var trait_type = _ref8.trait_type,
            values = _ref8.values;
          return {
            name: trait_type,
            values: values
          };
        }).filter(function (_ref9) {
          var name = _ref9.name;
          return !["Content Fabric Hash", "Total Minted Supply", "Creator"].includes(name);
        }));
      case 20:
      case "end":
        return _context4.stop();
    }
  }, _callee4, this);
}));

/**
 * <b><i>Requires login</i></b>
 *
 * Retrieve items owned by the specified or current user matching the specified parameters.
 *
 * @methodGroup User
 * @namedParams
 * @param {boolean=} includeFullMetadata=false - If true, will include full metadata for each item. Ignored if not logged in.
 * @param {string=} userAddress - Address of a user. If not specified, will return results for current user
 * @param {integer=} start=0 - PAGINATION: Index from which the results should start
 * @param {integer=} limit=50 - PAGINATION: Maximum number of results to return
 * @param {string=} sortBy="default" - Sort order. Options: `default`, `meta/display_name`
 * @param {boolean=} sortDesc=false - Sort results descending instead of ascending
 * @param {string=} filter - Filter results by item name.
 * @param {string=} contractAddress - Filter results by the address of the NFT contract
 * @param {string=} tokenId - Filter by token ID (if filtering by contract address)
 * @param {Object=} marketplaceParams - Filter results by marketplace
 * @param {Array<integer>=} collectionIndexes - If filtering by marketplace, filter by collection(s). The index refers to the index in the array `marketplace.collections`
 *
 * @returns {Promise<Object>} - Results of the query and pagination info
 */
exports.UserItems = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee5() {
  var _ref11,
    _ref11$sortBy,
    sortBy,
    _ref11$includeFullMet,
    includeFullMetadata,
    _args5 = arguments;
  return _regeneratorRuntime.wrap(function _callee5$(_context5) {
    while (1) switch (_context5.prev = _context5.next) {
      case 0:
        _ref11 = _args5.length > 0 && _args5[0] !== undefined ? _args5[0] : {}, _ref11$sortBy = _ref11.sortBy, sortBy = _ref11$sortBy === void 0 ? "default" : _ref11$sortBy, _ref11$includeFullMet = _ref11.includeFullMetadata, includeFullMetadata = _ref11$includeFullMet === void 0 ? false : _ref11$includeFullMet;
        if (!(includeFullMetadata && this.loggedIn)) {
          _context5.next = 5;
          break;
        }
        return _context5.abrupt("return", this.FilteredQuery(_objectSpread({
          mode: "owned-full-meta",
          sortBy: sortBy
        }, _args5[0] || {})));
      case 5:
        return _context5.abrupt("return", this.FilteredQuery(_objectSpread({
          mode: "owned",
          sortBy: sortBy
        }, _args5[0] || {})));
      case 6:
      case "end":
        return _context5.stop();
    }
  }, _callee5, this);
}));

/**
 * Return all listings for the current user. Not paginated.
 *
 * @methodGroup User
 * @namedParams
 * @param {string=} userAddress - Address of a user. If not specified, will return results for current user
 * @param {string=} sortBy="created" - Sort order. Options: `created`, `info/token_id`, `info/ordinal`, `price`, `nft/display_name`
 * @param {boolean=} sortDesc=false - Sort results descending instead of ascending
 * @param {Object=} marketplaceParams - Filter results by marketplace
 * @param {string=} contractAddress - Filter results by the address of the NFT contract
 * @param {string=} tokenId - Filter by token ID (if filtering by contract address)
 *
 * @returns {Promise<Array<Object>>} - List of current user's listings
 */
exports.UserListings = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee6() {
  var _ref13,
    userAddress,
    _ref13$sortBy,
    sortBy,
    _ref13$sortDesc,
    sortDesc,
    contractAddress,
    tokenId,
    marketplaceParams,
    _args6 = arguments;
  return _regeneratorRuntime.wrap(function _callee6$(_context6) {
    while (1) switch (_context6.prev = _context6.next) {
      case 0:
        _ref13 = _args6.length > 0 && _args6[0] !== undefined ? _args6[0] : {}, userAddress = _ref13.userAddress, _ref13$sortBy = _ref13.sortBy, sortBy = _ref13$sortBy === void 0 ? "created" : _ref13$sortBy, _ref13$sortDesc = _ref13.sortDesc, sortDesc = _ref13$sortDesc === void 0 ? false : _ref13$sortDesc, contractAddress = _ref13.contractAddress, tokenId = _ref13.tokenId, marketplaceParams = _ref13.marketplaceParams;
        _context6.next = 3;
        return this.FilteredQuery({
          mode: "listings",
          start: 0,
          limit: 10000,
          sortBy: sortBy,
          sortDesc: sortDesc,
          sellerAddress: userAddress || this.UserAddress(),
          marketplaceParams: marketplaceParams,
          contractAddress: contractAddress,
          tokenId: tokenId,
          includeCheckoutLocked: true
        });
      case 3:
        return _context6.abrupt("return", _context6.sent.results);
      case 4:
      case "end":
        return _context6.stop();
    }
  }, _callee6, this);
}));

/**
 * Return all sales for the current user. Not paginated.
 *
 * @methodGroup User
 * @namedParams
 * @param {string=} userAddress - Address of a user. If not specified, will return results for current user
 * @param {string=} sortBy="created" - Sort order. Options: `created`, `price`, `name`
 * @param {boolean=} sortDesc=false - Sort results descending instead of ascending
 * @param {Object=} marketplaceParams - Filter results by marketplace
 * @param {string=} contractAddress - Filter results by the address of the NFT contract
 * @param {string=} tokenId - Filter by token ID (if filtering by contract address)
 * @param {integer=} startTime - Filter by results listed after the specified time (in milliseconds since epoch)
 * @param {integer=} endTime - Filter by results listed before the specified time (in milliseconds since epoch)
 * @param {integer=} lastNDays - Filter by results listed in the past N days
 *
 * @returns {Promise<Array<Object>>} - List of current user's sales
 */
exports.UserSales = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee7() {
  var _ref15,
    userAddress,
    _ref15$sortBy,
    sortBy,
    _ref15$sortDesc,
    sortDesc,
    contractAddress,
    tokenId,
    startTime,
    endTime,
    lastNDays,
    marketplaceParams,
    _args7 = arguments;
  return _regeneratorRuntime.wrap(function _callee7$(_context7) {
    while (1) switch (_context7.prev = _context7.next) {
      case 0:
        _ref15 = _args7.length > 0 && _args7[0] !== undefined ? _args7[0] : {}, userAddress = _ref15.userAddress, _ref15$sortBy = _ref15.sortBy, sortBy = _ref15$sortBy === void 0 ? "created" : _ref15$sortBy, _ref15$sortDesc = _ref15.sortDesc, sortDesc = _ref15$sortDesc === void 0 ? false : _ref15$sortDesc, contractAddress = _ref15.contractAddress, tokenId = _ref15.tokenId, startTime = _ref15.startTime, endTime = _ref15.endTime, lastNDays = _ref15.lastNDays, marketplaceParams = _ref15.marketplaceParams;
        _context7.next = 3;
        return this.FilteredQuery({
          mode: "sales",
          start: 0,
          limit: 10000,
          sortBy: sortBy,
          sortDesc: sortDesc,
          sellerAddress: userAddress || this.UserAddress(),
          startTime: startTime,
          endTime: endTime,
          lastNDays: lastNDays,
          marketplaceParams: marketplaceParams,
          contractAddress: contractAddress,
          tokenId: tokenId
        });
      case 3:
        return _context7.abrupt("return", _context7.sent.results);
      case 4:
      case "end":
        return _context7.stop();
    }
  }, _callee7, this);
}));

/**
 * Return all transfers and sales for the current user. Not paginated.
 *
 * @methodGroup User
 * @namedParams
 * @param {string=} userAddress - Address of a user. If not specified, will return results for current user
 * @param {string=} sortBy="created" - Sort order. Options: `created`, `price`, `name`
 * @param {boolean=} sortDesc=false - Sort results descending instead of ascending
 * @param {Object=} marketplaceParams - Filter results by marketplace
 * @param {string=} contractAddress - Filter results by the address of the NFT contract
 * @param {string=} tokenId - Filter by token ID (if filtering by contract address)
 * @param {integer=} startTime - Filter by results listed after the specified time (in milliseconds since epoch)
 * @param {integer=} endTime - Filter by results listed before the specified time (in milliseconds since epoch)
 * @param {integer=} lastNDays - Filter by results listed in the past N days
 *
 * @returns {Promise<Array<Object>>} - List of current user's sales
 */
exports.UserTransfers = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee8() {
  var _ref17,
    userAddress,
    _ref17$sortBy,
    sortBy,
    _ref17$sortDesc,
    sortDesc,
    contractAddress,
    tokenId,
    startTime,
    endTime,
    lastNDays,
    marketplaceParams,
    _args8 = arguments;
  return _regeneratorRuntime.wrap(function _callee8$(_context8) {
    while (1) switch (_context8.prev = _context8.next) {
      case 0:
        _ref17 = _args8.length > 0 && _args8[0] !== undefined ? _args8[0] : {}, userAddress = _ref17.userAddress, _ref17$sortBy = _ref17.sortBy, sortBy = _ref17$sortBy === void 0 ? "created" : _ref17$sortBy, _ref17$sortDesc = _ref17.sortDesc, sortDesc = _ref17$sortDesc === void 0 ? false : _ref17$sortDesc, contractAddress = _ref17.contractAddress, tokenId = _ref17.tokenId, startTime = _ref17.startTime, endTime = _ref17.endTime, lastNDays = _ref17.lastNDays, marketplaceParams = _ref17.marketplaceParams;
        _context8.next = 3;
        return this.FilteredQuery({
          mode: "transfers",
          start: 0,
          limit: 10000,
          sortBy: sortBy,
          sortDesc: sortDesc,
          sellerAddress: userAddress || this.UserAddress(),
          marketplaceParams: marketplaceParams,
          contractAddress: contractAddress,
          tokenId: tokenId,
          startTime: startTime,
          endTime: endTime,
          lastNDays: lastNDays
        });
      case 3:
        return _context8.abrupt("return", _context8.sent.results);
      case 4:
      case "end":
        return _context8.stop();
    }
  }, _callee8, this);
}));

/* TENANT */

/**
 * Retrieve configuration information about the specified tenant, or the tenant associated with the specified contract.
 *
 * This information includes the royalty rate the tenant receives for secondary sales.
 *
 * @methodGroup Tenants
 * @namedParams
 * @param {string=} tenantId - The ID of the tenant for which to retrieve configuration
 * @param {string=} contractAddress - The ID of an nft contract for which to retrieve configuration
 *
 * @returns {Promise<Object>} - The tenant configuration
 */
exports.TenantConfiguration = /*#__PURE__*/function () {
  var _ref19 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee9(_ref18) {
    var tenantId, contractAddress;
    return _regeneratorRuntime.wrap(function _callee9$(_context9) {
      while (1) switch (_context9.prev = _context9.next) {
        case 0:
          tenantId = _ref18.tenantId, contractAddress = _ref18.contractAddress;
          _context9.prev = 1;
          contractAddress = contractAddress ? Utils.FormatAddress(contractAddress) : undefined;
          if (this.tenantConfigs[contractAddress || tenantId]) {
            _context9.next = 7;
            break;
          }
          _context9.next = 6;
          return Utils.ResponseToJson(this.client.authClient.MakeAuthServiceRequest({
            path: contractAddress ? UrlJoin("as", "config", "nft", contractAddress) : UrlJoin("as", "config", "tnt", tenantId),
            method: "GET"
          }));
        case 6:
          this.tenantConfigs[contractAddress || tenantId] = _context9.sent;
        case 7:
          return _context9.abrupt("return", this.tenantConfigs[contractAddress || tenantId]);
        case 10:
          _context9.prev = 10;
          _context9.t0 = _context9["catch"](1);
          this.Log("Failed to load tenant configuration", true, _context9.t0);
          return _context9.abrupt("return", {});
        case 14:
        case "end":
          return _context9.stop();
      }
    }, _callee9, this, [[1, 10]]);
  }));
  return function (_x2) {
    return _ref19.apply(this, arguments);
  };
}();

/**
 * Retrieve the current exchange rate for the specified currency to USD
 *
 * @methodGroup Tenants
 * @namedParams
 * @param {string} currency - The currency for which to retrieve the USD exchange rate
 */
exports.ExchangeRate = /*#__PURE__*/function () {
  var _ref21 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee10(_ref20) {
    var currency;
    return _regeneratorRuntime.wrap(function _callee10$(_context10) {
      while (1) switch (_context10.prev = _context10.next) {
        case 0:
          currency = _ref20.currency;
          if (currency) {
            _context10.next = 3;
            break;
          }
          throw Error("Eluvio Wallet Client: Invalid or missing currency in ExchangeRate");
        case 3:
          _context10.next = 5;
          return Utils.ResponseToJson(this.client.authClient.MakeAuthServiceRequest({
            path: UrlJoin("as", "xr", "ebanx", currency),
            method: "GET"
          }));
        case 5:
          return _context10.abrupt("return", _context10.sent);
        case 6:
        case "end":
          return _context10.stop();
      }
    }, _callee10, this);
  }));
  return function (_x3) {
    return _ref21.apply(this, arguments);
  };
}();

/**
 * Retrieve custom CSS for the specified tenant
 *
 * @methodGroup Tenants
 * @namedParams
 * @param {Object} tenantSlug
 *
 * @returns {Promise<string>} - The CSS of the tenant
 */
exports.TenantCSS = /*#__PURE__*/function () {
  var _ref23 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee11(_ref22) {
    var tenantSlug;
    return _regeneratorRuntime.wrap(function _callee11$(_context11) {
      while (1) switch (_context11.prev = _context11.next) {
        case 0:
          tenantSlug = _ref22.tenantSlug;
          if (this.cachedCSS[tenantSlug]) {
            _context11.next = 5;
            break;
          }
          _context11.next = 4;
          return this.client.ContentObjectMetadata({
            libraryId: this.mainSiteLibraryId,
            objectId: this.mainSiteId,
            metadataSubtree: UrlJoin("/public", "asset_metadata", "tenants", tenantSlug, "info", "branding", "wallet_css"),
            authorizationToken: this.publicStaticToken
          });
        case 4:
          this.cachedCSS[tenantSlug] = _context11.sent;
        case 5:
          return _context11.abrupt("return", this.cachedCSS[tenantSlug] || "");
        case 6:
        case "end":
          return _context11.stop();
      }
    }, _callee11, this);
  }));
  return function (_x4) {
    return _ref23.apply(this, arguments);
  };
}();

/* MARKETPLACE */

/**
 * Retrieve available stock for the specified marketplace, organized by SKU.
 *
 * If a user is logged in, stock information will also include how many of that item the user has purchased.
 *
 * @methodGroup Marketplaces
 * @namedParams
 * @param {Object} marketplaceParams - Parameters of the marketplace
 *
 * @returns {Promise<Object>} - Stock info for items in the marketplace
 */
exports.MarketplaceStock = /*#__PURE__*/function () {
  var _ref25 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee12(_ref24) {
    var marketplaceParams, tenantId, marketplaceInfo;
    return _regeneratorRuntime.wrap(function _callee12$(_context12) {
      while (1) switch (_context12.prev = _context12.next) {
        case 0:
          marketplaceParams = _ref24.marketplaceParams, tenantId = _ref24.tenantId;
          if (!tenantId) {
            marketplaceInfo = this.MarketplaceInfo({
              marketplaceParams: marketplaceParams
            });
            tenantId = marketplaceInfo.tenantId;
          }
          if (!this.loggedIn) {
            _context12.next = 6;
            break;
          }
          _context12.next = 5;
          return Utils.ResponseToJson(this.client.authClient.MakeAuthServiceRequest({
            path: UrlJoin("as", "wlt", "nft", "info", tenantId),
            method: "GET",
            headers: {
              Authorization: "Bearer ".concat(this.AuthToken())
            }
          }));
        case 5:
          return _context12.abrupt("return", _context12.sent);
        case 6:
          _context12.next = 8;
          return Utils.ResponseToJson(this.client.authClient.MakeAuthServiceRequest({
            path: UrlJoin("as", "nft", "stock", tenantId),
            method: "GET"
          }));
        case 8:
          return _context12.abrupt("return", _context12.sent);
        case 9:
        case "end":
          return _context12.stop();
      }
    }, _callee12, this);
  }));
  return function (_x5) {
    return _ref25.apply(this, arguments);
  };
}();

/**
 * Retrieve basic information about a specific available marketplace with the specified tenant/marketplace slug, ID, or hash.
 *
 * Includes the slugs, ID and hash of the marketplace, as well as branding information.
 *
 * To retrieve full metadata for the marketplace, use the <a href="#.Marketplace">Marketplace</a> method.
 *
 * @methodGroup Marketplaces
 * @namedParams
 * @param {Object} marketplaceParams - Parameters of the marketplace
 *
 * @returns {Promise<Object>} - Info about the marketplace
 */
exports.MarketplaceInfo = function (_ref26) {
  var marketplaceParams = _ref26.marketplaceParams;
  var _ref27 = marketplaceParams || {},
    tenantSlug = _ref27.tenantSlug,
    marketplaceSlug = _ref27.marketplaceSlug,
    marketplaceId = _ref27.marketplaceId,
    marketplaceHash = _ref27.marketplaceHash;
  var marketplaceInfo;
  if (tenantSlug && marketplaceSlug) {
    marketplaceInfo = (this.availableMarketplaces[tenantSlug] || {})[marketplaceSlug];
  } else {
    marketplaceId = marketplaceHash ? this.client.utils.DecodeVersionHash(marketplaceHash).objectId : marketplaceId;
    marketplaceInfo = this.availableMarketplacesById[marketplaceId];
  }
  if (!marketplaceInfo) {
    throw Error("Eluvio Wallet Client: Unable to find marketplace with parameters ".concat(JSON.stringify(arguments)));
  }
  return marketplaceInfo;
};

/**
 * Retrieve custom CSS for the specified marketplace
 *
 * @methodGroup Marketplaces
 * @namedParams
 * @param {Object} marketplaceParams - Parameters of the marketplace
 *
 * @returns {Promise<string>} - The CSS of the marketplace
 */
exports.MarketplaceCSS = /*#__PURE__*/function () {
  var _ref29 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee13(_ref28) {
    var marketplaceParams, marketplaceInfo, marketplaceHash;
    return _regeneratorRuntime.wrap(function _callee13$(_context13) {
      while (1) switch (_context13.prev = _context13.next) {
        case 0:
          marketplaceParams = _ref28.marketplaceParams;
          marketplaceInfo = this.MarketplaceInfo({
            marketplaceParams: marketplaceParams
          });
          marketplaceHash = marketplaceInfo.marketplaceHash;
          if (this.cachedCSS[marketplaceHash]) {
            _context13.next = 7;
            break;
          }
          _context13.next = 6;
          return this.client.ContentObjectMetadata({
            versionHash: marketplaceHash,
            metadataSubtree: "public/asset_metadata/info/branding/custom_css",
            authorizationToken: this.publicStaticToken,
            noAuth: true
          });
        case 6:
          this.cachedCSS[marketplaceHash] = _context13.sent;
        case 7:
          return _context13.abrupt("return", this.cachedCSS[marketplaceHash] || "");
        case 8:
        case "end":
          return _context13.stop();
      }
    }, _callee13, this);
  }));
  return function (_x6) {
    return _ref29.apply(this, arguments);
  };
}();

/**
 * Retrieve info about all available marketplaces
 *
 * @methodGroup Marketplaces
 * @namedParams
 * @param {boolean=} organizeById - By default, the returned marketplace info is organized by tenant and marketplace slug. If this option is enabled, the marketplaces will be organized by marketplace ID instead.
 * @param {boolean=} forceReload=false - If specified, a new request will be made to check the currently available marketplaces instead of returning cached info
 *
 * @returns {Promise<Object>} - Info about available marketplaces
 */
exports.AvailableMarketplaces = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee14() {
  var _ref31,
    organizeById,
    _ref31$forceReload,
    forceReload,
    _args14 = arguments;
  return _regeneratorRuntime.wrap(function _callee14$(_context14) {
    while (1) switch (_context14.prev = _context14.next) {
      case 0:
        _ref31 = _args14.length > 0 && _args14[0] !== undefined ? _args14[0] : {}, organizeById = _ref31.organizeById, _ref31$forceReload = _ref31.forceReload, forceReload = _ref31$forceReload === void 0 ? false : _ref31$forceReload;
        if (!forceReload) {
          _context14.next = 4;
          break;
        }
        _context14.next = 4;
        return this.LoadAvailableMarketplaces(true);
      case 4:
        return _context14.abrupt("return", _objectSpread({}, organizeById ? this.availableMarketplacesById : this.availableMarketplaces));
      case 5:
      case "end":
        return _context14.stop();
    }
  }, _callee14, this);
}));

/**
 * Retrieve full information about the specified marketplace
 *
 * <b><i>Note</i></b> - Upon changing login state, the marketplace should be retrieved again as permission info in marketplace items may be different depending on the current user's permissions.
 *
 * @methodGroup Marketplaces
 * @namedParams
 * @param {Object} marketplaceParams - Parameters of the marketplace
 *
 * @returns {Promise<Object>} - The full information for the marketplace
 */
exports.Marketplace = /*#__PURE__*/function () {
  var _ref33 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee15(_ref32) {
    var marketplaceParams;
    return _regeneratorRuntime.wrap(function _callee15$(_context15) {
      while (1) switch (_context15.prev = _context15.next) {
        case 0:
          marketplaceParams = _ref32.marketplaceParams;
          return _context15.abrupt("return", this.LoadMarketplace(marketplaceParams));
        case 2:
        case "end":
          return _context15.stop();
      }
    }, _callee15, this);
  }));
  return function (_x7) {
    return _ref33.apply(this, arguments);
  };
}();

/* NFTS */

/**
 * Return info about the specified NFT contract, including the cap, current total supply, and total minted and burned.
 *
 * @methodGroup NFTs
 * @namedParams
 * @param {string} contractAddress - The contract address of the NFT
 *
 * @returns {Promise<Object>} - Information about the specified contract
 */
exports.NFTContractStats = /*#__PURE__*/function () {
  var _ref35 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee16(_ref34) {
    var contractAddress;
    return _regeneratorRuntime.wrap(function _callee16$(_context16) {
      while (1) switch (_context16.prev = _context16.next) {
        case 0:
          contractAddress = _ref34.contractAddress;
          _context16.next = 3;
          return Utils.ResponseToJson(this.client.authClient.MakeAuthServiceRequest({
            path: UrlJoin("as", "nft", "info", contractAddress),
            method: "GET"
          }));
        case 3:
          return _context16.abrupt("return", _context16.sent);
        case 4:
        case "end":
          return _context16.stop();
      }
    }, _callee16, this);
  }));
  return function (_x8) {
    return _ref35.apply(this, arguments);
  };
}();

/**
 * Load full info for the specified NFT
 *
 * @methodGroup NFTs
 * @namedParams
 * @param {string} contractAddress - The contract address of the NFT
 * @param {string} tokenId - The token ID of the NFT
 */
exports.NFT = /*#__PURE__*/function () {
  var _ref37 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee17(_ref36) {
    var tokenId, contractAddress, nft, assetMetadata, localizedMetadata;
    return _regeneratorRuntime.wrap(function _callee17$(_context17) {
      while (1) switch (_context17.prev = _context17.next) {
        case 0:
          tokenId = _ref36.tokenId, contractAddress = _ref36.contractAddress;
          _context17.t0 = FormatNFTDetails;
          _context17.next = 4;
          return Utils.ResponseToJson(this.client.authClient.MakeAuthServiceRequest({
            path: UrlJoin("as", "nft", "info", contractAddress, tokenId),
            method: "GET"
          }));
        case 4:
          _context17.t1 = _context17.sent;
          nft = (0, _context17.t0)(_context17.t1);
          _context17.next = 8;
          return this.client.ContentObjectMetadata({
            versionHash: nft.details.VersionHash,
            metadataSubtree: "public/asset_metadata/nft",
            produceLinkUrls: true
          });
        case 8:
          _context17.t2 = _context17.sent;
          if (_context17.t2) {
            _context17.next = 11;
            break;
          }
          _context17.t2 = {};
        case 11:
          assetMetadata = _context17.t2;
          nft.metadata = MergeWith({}, assetMetadata, nft.metadata, function (a, b) {
            return b === null || b === "" ? a : undefined;
          });
          if (!this.localization) {
            _context17.next = 21;
            break;
          }
          _context17.next = 16;
          return this.client.ContentObjectMetadata({
            versionHash: nft.details.VersionHash,
            metadataSubtree: UrlJoin("public", "asset_metadata", "localizations", this.localization, "nft"),
            produceLinkUrls: true
          });
        case 16:
          _context17.t3 = _context17.sent;
          if (_context17.t3) {
            _context17.next = 19;
            break;
          }
          _context17.t3 = {};
        case 19:
          localizedMetadata = _context17.t3;
          nft.metadata = MergeWith({}, nft.metadata, localizedMetadata, function (a, b) {
            return b === null || b === "" ? a : undefined;
          });
        case 21:
          _context17.next = 23;
          return this.TenantConfiguration({
            contractAddress: contractAddress
          });
        case 23:
          nft.config = _context17.sent;
          return _context17.abrupt("return", FormatNFTMetadata(this, nft));
        case 25:
        case "end":
          return _context17.stop();
      }
    }, _callee17, this);
  }));
  return function (_x9) {
    return _ref37.apply(this, arguments);
  };
}();

/**
 * <b><i>Requires login</i></b>
 *
 * Transfer the specified NFT owned by the current user to the specified address
 *
 * @methodGroup NFTs
 * @namedParams
 * @param {string} contractAddress - The contract address of the NFT
 * @param {string} tokenId - The token ID of the NFT
 * @param {string} targetAddress - The address to which to transfer the NFT
 */
exports.TransferNFT = /*#__PURE__*/function () {
  var _ref39 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee18(_ref38) {
    var contractAddress, tokenId, targetAddress;
    return _regeneratorRuntime.wrap(function _callee18$(_context18) {
      while (1) switch (_context18.prev = _context18.next) {
        case 0:
          contractAddress = _ref38.contractAddress, tokenId = _ref38.tokenId, targetAddress = _ref38.targetAddress;
          if (!(!targetAddress || !Utils.ValidAddress(targetAddress))) {
            _context18.next = 3;
            break;
          }
          throw Error("Eluvio Wallet Client: Invalid or missing target address in UserTransferNFT");
        case 3:
          _context18.next = 5;
          return this.client.authClient.MakeAuthServiceRequest({
            path: UrlJoin("as", "wlt", "mkt", "xfer"),
            method: "POST",
            body: {
              contract: Utils.FormatAddress(contractAddress),
              token: tokenId,
              to_addr: Utils.FormatAddress(targetAddress)
            },
            headers: {
              Authorization: "Bearer ".concat(this.AuthToken())
            }
          });
        case 5:
          return _context18.abrupt("return", _context18.sent);
        case 6:
        case "end":
          return _context18.stop();
      }
    }, _callee18, this);
  }));
  return function (_x10) {
    return _ref39.apply(this, arguments);
  };
}();

/** LISTINGS */

/**
 * Retrieve the status of the specified listing
 *
 * @methodGroup Listings
 * @namedParams
 * @param {string=} listingId - The ID of the listing
 *
 * @returns {Promise<Object>} - The status of the listing
 */
exports.ListingStatus = /*#__PURE__*/function () {
  var _ref41 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee19(_ref40) {
    var listingId;
    return _regeneratorRuntime.wrap(function _callee19$(_context19) {
      while (1) switch (_context19.prev = _context19.next) {
        case 0:
          listingId = _ref40.listingId;
          _context19.prev = 1;
          _context19.t0 = Utils;
          _context19.next = 5;
          return this.client.authClient.MakeAuthServiceRequest({
            path: UrlJoin("as", "mkt", "status", listingId),
            method: "GET"
          });
        case 5:
          _context19.t1 = _context19.sent;
          _context19.next = 8;
          return _context19.t0.ResponseToJson.call(_context19.t0, _context19.t1);
        case 8:
          return _context19.abrupt("return", _context19.sent);
        case 11:
          _context19.prev = 11;
          _context19.t2 = _context19["catch"](1);
          if (!(_context19.t2.status === 404)) {
            _context19.next = 15;
            break;
          }
          return _context19.abrupt("return");
        case 15:
          throw _context19.t2;
        case 16:
        case "end":
          return _context19.stop();
      }
    }, _callee19, this, [[1, 11]]);
  }));
  return function (_x11) {
    return _ref41.apply(this, arguments);
  };
}();

/**
 * Retrieve a specific listing
 *
 * NOTE: When a listing is sold or deleted, it will no longer be queryable with this API. Use <a href="#.ListingStatus">ListingStatus</a> instead.
 *
 * @methodGroup Listings
 * @namedParams
 * @param {string=} listingId - The ID of the listing
 *
 * @returns {Promise<Object>} - The listing
 */
exports.Listing = /*#__PURE__*/function () {
  var _ref43 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee20(_ref42) {
    var listingId;
    return _regeneratorRuntime.wrap(function _callee20$(_context20) {
      while (1) switch (_context20.prev = _context20.next) {
        case 0:
          listingId = _ref42.listingId;
          _context20.t0 = FormatNFT;
          _context20.t1 = this;
          _context20.t2 = Utils;
          _context20.next = 6;
          return this.client.authClient.MakeAuthServiceRequest({
            path: UrlJoin("as", "mkt", "l", listingId),
            method: "GET"
          });
        case 6:
          _context20.t3 = _context20.sent;
          _context20.next = 9;
          return _context20.t2.ResponseToJson.call(_context20.t2, _context20.t3);
        case 9:
          _context20.t4 = _context20.sent;
          return _context20.abrupt("return", (0, _context20.t0)(_context20.t1, _context20.t4));
        case 11:
        case "end":
          return _context20.stop();
      }
    }, _callee20, this);
  }));
  return function (_x12) {
    return _ref43.apply(this, arguments);
  };
}();

/**
 * Retrieve listings matching the specified parameters.
 *
 * @methodGroup Listings
 * @namedParams
 * @param {integer=} start=0 - PAGINATION: Index from which the results should start
 * @param {integer=} limit=50 - PAGINATION: Maximum number of results to return
 * @param {string=} sortBy="created" - Sort order. Options: `created`, `info/token_id`, `info/ordinal`, `price`, `nft/display_name`
 * @param {boolean=} sortDesc=false - Sort results descending instead of ascending
 * @param {string=} filter - Filter results by item name.
 *  <br /><br />
 *  NOTE: This string must be an <b>exact match</b> on the item name.
 * You can retrieve all available item names from the <a href="#.ListingNames">ListingNames method</a>.
 *  @param {Array<string>=} editionFilters - Filter results by item edition.
 *  <br /><br />
 *  NOTE: This string must be an <b>exact match</b> on the edition name.
 * You can retrieve all available item edition names from the <a href="#.ListingEditionNames">ListingEditionNames method</a>.
 *  @param {Array<Object>} attributeFilters - Filter results by item attributes. Each entry should include name and value (e.g. `[{name: "attribute-name", value: "attribute-value"}]`)
 *  <br /><br />
 *  NOTE: These filters must be an <b>exact match</b> on the attribute name and value.
 * You can retrieve all available item attributes from the <a href="#.ListingAttributes">ListingAttributes method</a>.
 * @param {Object=} priceRange - Filter min and/or max price (e.g. `{min: 1}` `{max: 2}` `{min: 1.50, max: 10.50})
 * @param {string=} sellerAddress - Filter by a specific seller
 * @param {string=} contractAddress - Filter results by the address of the NFT contract
 * @param {string=} tokenId - Filter by token ID (if filtering by contract address)
 * @param {string=} currency - Filter results by purchase currency. Available options: `usdc`
 * @param {Object=} marketplaceParams - Filter results by marketplace
 * @param {Array<integer>=} collectionIndexes - If filtering by marketplace, filter by collection(s). The index refers to the index in the array `marketplace.collections`
 * @param {integer=} startTime - Filter by results listed after the specified time (in milliseconds since epoch)
 * @param {integer=} endTime - Filter by results listed before the specified time (in milliseconds since epoch)
 * @param {integer=} lastNDays - Filter by results listed in the past N days
 * @param {boolean=} includeCheckoutLocked - If specified, listings which are currently in the checkout process (and not so currently purchasable) will be included in the results. By default they are excluded.
 *
 * @returns {Promise<Object>} - Results of the query and pagination info
 */
exports.Listings = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee21() {
  var _args21 = arguments;
  return _regeneratorRuntime.wrap(function _callee21$(_context21) {
    while (1) switch (_context21.prev = _context21.next) {
      case 0:
        return _context21.abrupt("return", this.FilteredQuery(_objectSpread({
          mode: "listings"
        }, _args21[0] || {})));
      case 1:
      case "end":
        return _context21.stop();
    }
  }, _callee21, this);
}));

/**
 * Retrieve stats for listings matching the specified parameters.
 *
 * @methodGroup Listings
 * @namedParams
 * @param {integer=} start=0 - PAGINATION: Index from which the results should start
 * @param {integer=} limit=50 - PAGINATION: Maximum number of results to return
 * @param {string=} sortBy="created" - Sort order. Options: `created`, `info/token_id`, `info/ordinal`, `price`, `nft/display_name`
 * @param {boolean=} sortDesc=false - Sort results descending instead of ascending
 * @param {string=} filter - Filter results by item name.
 *  <br /><br />
 *  NOTE: This string must be an <b>exact match</b> on the item name.
 * You can retrieve all available item names from the <a href="#.ListingNames">ListingNames method</a>.
 *  @param {Array<string>} editionFilters - Filter results by item edition.
 *  <br /><br />
 *  NOTE: This string must be an <b>exact match</b> on the edition name.
 * You can retrieve all available item edition names from the <a href="#.ListingEditionNames">ListingEditionNames method</a>.
 *  @param {Array<Object>} attributeFilters - Filter results by item attributes. Each entry should include name and value (e.g. `[{name: "attribute-name", value: "attribute-value"}]`)
 *  <br /><br />
 *  NOTE: These filters must be an <b>exact match</b> on the attribute name and value.
 * You can retrieve all available item attributes from the <a href="#.ListingAttributes">ListingAttributes method</a>.
 * @param {Object=} priceRange - Filter min and/or max price (e.g. `{min: 1}` `{max: 2}` `{min: 1.50, max: 10.50})
 * @param {string=} sellerAddress - Filter by a specific seller
 * @param {string=} contractAddress - Filter results by the address of the NFT contract
 * @param {string=} tokenId - Filter by token ID (if filtering by contract address)
 * @param {string=} currency - Filter results by purchase currency. Available options: `usdc`
 * @param {Object=} marketplaceParams - Filter results by marketplace
 * @param {Array<integer>=} collectionIndexes - If filtering by marketplace, filter by collection(s). The index refers to the index in the array `marketplace.collections`
 * @param {integer=} startTime - Filter by results listed after the specified time (in milliseconds since epoch)
 * @param {integer=} endTime - Filter by results listed before the specified time (in milliseconds since epoch)
 * @param {integer=} lastNDays - Filter by results listed in the past N days
 *
 * @returns {Promise<Object>} - Statistics about listings. All prices in USD.
 */
exports.ListingStats = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee22() {
  var _args22 = arguments;
  return _regeneratorRuntime.wrap(function _callee22$(_context22) {
    while (1) switch (_context22.prev = _context22.next) {
      case 0:
        return _context22.abrupt("return", this.FilteredQuery(_objectSpread({
          mode: "listing-stats"
        }, _args22[0] || {})));
      case 1:
      case "end":
        return _context22.stop();
    }
  }, _callee22, this);
}));

/**
 * Retrieve sales matching the specified parameters.
 *
 * @methodGroup Listings
 * @namedParams
 * @param {integer=} start=0 - PAGINATION: Index from which the results should start
 * @param {integer=} limit=50 - PAGINATION: Maximum number of results to return
 * @param {string=} sortBy="created" - Sort order. Options: `created`, `price`, `name`
 * @param {boolean=} sortDesc=false - Sort results descending instead of ascending
 * @param {string=} filter - Filter results by item name.
 *  <br /><br />
 *  NOTE: This string must be an <b>exact match</b> on the item name.
 * You can retrieve all available item names from the <a href="#.ListingNames">ListingNames method</a>.
 *  @param {Array<string>} editionFilters - Filter results by item edition.
 *  <br /><br />
 *  NOTE: This string must be an <b>exact match</b> on the edition name.
 * You can retrieve all available item edition names from the <a href="#.ListingEditionNames">ListingEditionNames method</a>.
 *  @param {Array<Object>} attributeFilters - Filter results by item attributes. Each entry should include name and value (e.g. `[{name: "attribute-name", value: "attribute-value"}]`)
 *  <br /><br />
 *  NOTE: These filters must be an <b>exact match</b> on the attribute name and value.
 * You can retrieve all available item attributes from the <a href="#.ListingAttributes">ListingAttributes method</a>.
 * @param {string=} sellerAddress - Filter by a specific seller
 * @param {string=} contractAddress - Filter results by the address of the NFT contract
 * @param {string=} tokenId - Filter by token ID (if filtering by contract address)
 * @param {string=} currency - Filter results by purchase currency. Available options: `usdc`
 * @param {Object=} marketplaceParams - Filter results by marketplace
 * @param {Array<integer>=} collectionIndexes - If filtering by marketplace, filter by collection(s). The index refers to the index in the array `marketplace.collections`
 * @param {integer=} startTime - Filter by results listed after the specified time (in milliseconds since epoch)
 * @param {integer=} endTime - Filter by results listed before the specified time (in milliseconds since epoch)
 * @param {integer=} lastNDays - Filter by results listed in the past N days
 *
 * @returns {Promise<Object>} - Results of the query and pagination info
 */
exports.Sales = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee23() {
  var _args23 = arguments;
  return _regeneratorRuntime.wrap(function _callee23$(_context23) {
    while (1) switch (_context23.prev = _context23.next) {
      case 0:
        return _context23.abrupt("return", this.FilteredQuery(_objectSpread({
          mode: "sales"
        }, _args23[0] || {})));
      case 1:
      case "end":
        return _context23.stop();
    }
  }, _callee23, this);
}));

/**
 * Retrieve sales and transfers matching the specified parameters.
 *
 * @methodGroup Listings
 * @namedParams
 * @param {integer=} start=0 - PAGINATION: Index from which the results should start
 * @param {integer=} limit=50 - PAGINATION: Maximum number of results to return
 * @param {string=} sortBy="created" - Sort order. Options: `created`, `price`, `name`
 * @param {boolean=} sortDesc=false - Sort results descending instead of ascending
 * @param {string=} filter - Filter results by item name.
 *  <br /><br />
 *  NOTE: This string must be an <b>exact match</b> on the item name.
 * You can retrieve all available item names from the <a href="#.ListingNames">ListingNames method</a>.
 *  @param {Array<string>} editionFilters - Filter results by item edition.
 *  <br /><br />
 *  NOTE: This string must be an <b>exact match</b> on the edition name.
 * You can retrieve all available item edition names from the <a href="#.ListingEditionNames">ListingEditionNames method</a>.
 *  @param {Array<Object>} attributeFilters - Filter results by item attributes. Each entry should include name and value (e.g. `[{name: "attribute-name", value: "attribute-value"}]`)
 *  <br /><br />
 *  NOTE: These filters must be an <b>exact match</b> on the attribute name and value.
 * You can retrieve all available item attributes from the <a href="#.ListingAttributes">ListingAttributes method</a>.
 * @param {string=} sellerAddress - Filter by a specific seller
 * @param {string=} contractAddress - Filter results by the address of the NFT contract
 * @param {string=} tokenId - Filter by token ID (if filtering by contract address)
 * @param {string=} currency - Filter results by purchase currency. Available options: `usdc`
 * @param {Object=} marketplaceParams - Filter results by marketplace
 * @param {Array<integer>=} collectionIndexes - If filtering by marketplace, filter by collection(s). The index refers to the index in the array `marketplace.collections`
 * @param {integer=} startTime - Filter by results listed after the specified time (in milliseconds since epoch)
 * @param {integer=} endTime - Filter by results listed before the specified time (in milliseconds since epoch)
 * @param {integer=} lastNDays - Filter by results listed in the past N days
 *
 * @returns {Promise<Object>} - Results of the query and pagination info
 */
exports.Transfers = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee24() {
  var _args24 = arguments;
  return _regeneratorRuntime.wrap(function _callee24$(_context24) {
    while (1) switch (_context24.prev = _context24.next) {
      case 0:
        return _context24.abrupt("return", this.FilteredQuery(_objectSpread({
          mode: "transfers"
        }, _args24[0] || {})));
      case 1:
      case "end":
        return _context24.stop();
    }
  }, _callee24, this);
}));

/**
 * Retrieve stats for listings matching the specified parameters.
 *
 * @methodGroup Listings
 * @namedParams
 * @param {integer=} start=0 - PAGINATION: Index from which the results should start
 * @param {integer=} limit=50 - PAGINATION: Maximum number of results to return
 * @param {string=} sortBy="created" -
 * @param {boolean=} sortDesc=false - Sort results descending instead of ascending
 * @param {string=} filter - Filter results by item name.
 *  <br /><br />
 *  NOTE: This string must be an <b>exact match</b> on the item name.
 * You can retrieve all available item names from the <a href="#.ListingNames">ListingNames method</a>.
 *  @param {Array<string>} editionFilters - Filter results by item edition.
 *  <br /><br />
 *  NOTE: This string must be an <b>exact match</b> on the edition name.
 * You can retrieve all available item edition names from the <a href="#.ListingEditionNames">ListingEditionNames method</a>.
 *  @param {Array<Object>} attributeFilters - Filter results by item attributes. Each entry should include name and value (e.g. `[{name: "attribute-name", value: "attribute-value"}]`)
 *  <br /><br />
 *  NOTE: These filters must be an <b>exact match</b> on the attribute name and value.
 * You can retrieve all available item attributes from the <a href="#.ListingAttributes">ListingAttributes method</a>.
 * @param {string=} sellerAddress - Filter by a specific seller
 * @param {string=} contractAddress - Filter results by the address of the NFT contract
 * @param {string=} tokenId - Filter by token ID (if filtering by contract address)
 * @param {string=} currency - Filter results by purchase currency. Available options: `usdc`
 * @param {Object=} marketplaceParams - Filter results by marketplace
 * @param {Array<integer>=} collectionIndexes - If filtering by marketplace, filter by collection(s). The index refers to the index in the array `marketplace.collections`
 * @param {integer=} startTime - Filter by results listed after the specified time (in milliseconds since epoch)
 * @param {integer=} endTime - Filter by results listed before the specified time (in milliseconds since epoch)
 * @param {integer=} lastNDays - Filter by results listed in the past N days
 *
 * @returns {Promise<Object>} - Statistics about sales. All prices in USD.
 */
exports.SalesStats = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee25() {
  var _args25 = arguments;
  return _regeneratorRuntime.wrap(function _callee25$(_context25) {
    while (1) switch (_context25.prev = _context25.next) {
      case 0:
        return _context25.abrupt("return", this.FilteredQuery(_objectSpread({
          mode: "sales-stats"
        }, _args25[0] || {})));
      case 1:
      case "end":
        return _context25.stop();
    }
  }, _callee25, this);
}));

/**
 * Get the leaderboard rankings for the specified marketplace. If user address is specified, will return the ranking for the specified user (if present)
 *
 * @methodGroup Leaderboard
 * @namedParams
 * @param {Object=} marketplaceParams - Filter results by marketplace
 * @param {string=} userAddress - Retrieve the ranking for a specific user
 * @param {integer=} start=0 - PAGINATION: Index from which the results should start
 * @param {integer=} limit=50 - PAGINATION: Maximum number of results to return
 *
 * @returns {Promise<Array|Object>} - Returns a list of leaderboard rankings or, if userAddress is specified, ranking for that user.
 */
exports.Leaderboard = /*#__PURE__*/function () {
  var _ref50 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee26(_ref49) {
    var userAddress,
      marketplaceParams,
      params,
      _args26 = arguments;
    return _regeneratorRuntime.wrap(function _callee26$(_context26) {
      while (1) switch (_context26.prev = _context26.next) {
        case 0:
          userAddress = _ref49.userAddress, marketplaceParams = _ref49.marketplaceParams;
          if (!userAddress) {
            _context26.next = 20;
            break;
          }
          params = {
            addr: Utils.FormatAddress(userAddress)
          };
          if (!marketplaceParams) {
            _context26.next = 10;
            break;
          }
          _context26.t0 = "tenant:eq:";
          _context26.next = 7;
          return this.MarketplaceInfo({
            marketplaceParams: marketplaceParams
          });
        case 7:
          _context26.t1 = _context26.sent.tenantId;
          _context26.t2 = _context26.t0.concat.call(_context26.t0, _context26.t1);
          params.filter = [_context26.t2];
        case 10:
          _context26.t4 = Utils;
          _context26.next = 13;
          return this.client.authClient.MakeAuthServiceRequest({
            path: UrlJoin("as", "wlt", "ranks"),
            method: "GET",
            queryParams: params
          });
        case 13:
          _context26.t5 = _context26.sent;
          _context26.next = 16;
          return _context26.t4.ResponseToJson.call(_context26.t4, _context26.t5);
        case 16:
          _context26.t3 = _context26.sent;
          if (_context26.t3) {
            _context26.next = 19;
            break;
          }
          _context26.t3 = [];
        case 19:
          return _context26.abrupt("return", _context26.t3[0]);
        case 20:
          return _context26.abrupt("return", this.FilteredQuery(_objectSpread({
            mode: "leaderboard"
          }, _args26[0] || {})));
        case 21:
        case "end":
          return _context26.stop();
      }
    }, _callee26, this);
  }));
  return function (_x13) {
    return _ref50.apply(this, arguments);
  };
}();

/**
 * <b><i>Requires login</i></b>
 *
 * Create or update a listing for the specified item
 *
 * @methodGroup Listings
 * @namedParams
 * @param {string} contractAddress - The NFT contract address of the item
 * @param {string} tokenId - The token ID of the item
 * @param {number} price - The price of the listing, in USD
 * @param {string=} listingId - (When editing a listing) The ID of the existing listing
 *
 * @returns {Promise<string>} - The listing ID of the created listing
 */
exports.CreateListing = /*#__PURE__*/function () {
  var _ref52 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee27(_ref51) {
    var contractAddress, tokenId, price, listingId;
    return _regeneratorRuntime.wrap(function _callee27$(_context27) {
      while (1) switch (_context27.prev = _context27.next) {
        case 0:
          contractAddress = _ref51.contractAddress, tokenId = _ref51.tokenId, price = _ref51.price, listingId = _ref51.listingId;
          contractAddress = Utils.FormatAddress(contractAddress);
          if (!listingId) {
            _context27.next = 12;
            break;
          }
          _context27.t0 = Utils;
          _context27.next = 6;
          return this.client.authClient.MakeAuthServiceRequest({
            path: UrlJoin("as", "wlt", "mkt"),
            method: "PUT",
            body: {
              id: listingId,
              price: parseFloat(price)
            },
            headers: {
              Authorization: "Bearer ".concat(this.AuthToken())
            }
          });
        case 6:
          _context27.t1 = _context27.sent;
          _context27.next = 9;
          return _context27.t0.ResponseToFormat.call(_context27.t0, "text", _context27.t1);
        case 9:
          return _context27.abrupt("return", _context27.sent);
        case 12:
          _context27.t2 = Utils;
          _context27.next = 15;
          return this.client.authClient.MakeAuthServiceRequest({
            path: UrlJoin("as", "wlt", "mkt"),
            method: "POST",
            body: {
              contract: contractAddress,
              token: tokenId,
              price: parseFloat(price)
            },
            headers: {
              Authorization: "Bearer ".concat(this.AuthToken())
            }
          });
        case 15:
          _context27.t3 = _context27.sent;
          _context27.next = 18;
          return _context27.t2.ResponseToJson.call(_context27.t2, _context27.t3);
        case 18:
          return _context27.abrupt("return", _context27.sent);
        case 19:
        case "end":
          return _context27.stop();
      }
    }, _callee27, this);
  }));
  return function (_x14) {
    return _ref52.apply(this, arguments);
  };
}();

/**
 * <b><i>Requires login</i></b>
 *
 * Remove the specified listing
 *
 * @methodGroup Listings
 * @namedParams
 * @param {string} listingId - The ID of the listing to remove
 */
exports.RemoveListing = /*#__PURE__*/function () {
  var _ref54 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee28(_ref53) {
    var listingId;
    return _regeneratorRuntime.wrap(function _callee28$(_context28) {
      while (1) switch (_context28.prev = _context28.next) {
        case 0:
          listingId = _ref53.listingId;
          _context28.next = 3;
          return this.client.authClient.MakeAuthServiceRequest({
            path: UrlJoin("as", "wlt", "mkt", listingId),
            method: "DELETE",
            headers: {
              Authorization: "Bearer ".concat(this.AuthToken())
            }
          });
        case 3:
        case "end":
          return _context28.stop();
      }
    }, _callee28, this);
  }));
  return function (_x15) {
    return _ref54.apply(this, arguments);
  };
}();

/**
 * Retrieve all valid names for filtering listing sales names. Full item names are required for filtering sales results by name.
 *
 * Specify marketplace information to filter the results to only items offered in that marketplace.
 *
 * @methodGroup Listings
 * @namedParams
 * @param {Object} marketplaceParams - Parameters of a marketplace to filter results by
 *
 * @returns {Promise<Array<String>>} - A list of item names
 */
exports.SalesNames = /*#__PURE__*/function () {
  var _ref56 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee29(_ref55) {
    var marketplaceParams, tenantId;
    return _regeneratorRuntime.wrap(function _callee29$(_context29) {
      while (1) switch (_context29.prev = _context29.next) {
        case 0:
          marketplaceParams = _ref55.marketplaceParams;
          if (!marketplaceParams) {
            _context29.next = 5;
            break;
          }
          _context29.next = 4;
          return this.MarketplaceInfo({
            marketplaceParams: marketplaceParams
          });
        case 4:
          tenantId = _context29.sent.tenantId;
        case 5:
          _context29.t0 = Utils;
          _context29.next = 8;
          return this.client.authClient.MakeAuthServiceRequest({
            path: UrlJoin("as", "mkt", "names", "hst"),
            method: "GET",
            queryParams: tenantId ? {
              filter: "tenant:eq:".concat(tenantId)
            } : {}
          });
        case 8:
          _context29.t1 = _context29.sent;
          _context29.next = 11;
          return _context29.t0.ResponseToJson.call(_context29.t0, _context29.t1);
        case 11:
          return _context29.abrupt("return", _context29.sent);
        case 12:
        case "end":
          return _context29.stop();
      }
    }, _callee29, this);
  }));
  return function (_x16) {
    return _ref56.apply(this, arguments);
  };
}();

/**
 * Retrieve all valid names for filtering listings. Full item names are required for filtering listing results by name.
 *
 * Specify marketplace information to filter the results to only items offered in that marketplace.
 *
 * @methodGroup Listings
 * @namedParams
 * @param {Object} marketplaceParams - Parameters of a marketplace to filter results by
 *
 * @returns {Promise<Array<String>>} - A list of item names
 */
exports.ListingNames = /*#__PURE__*/function () {
  var _ref58 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee30(_ref57) {
    var marketplaceParams, tenantId;
    return _regeneratorRuntime.wrap(function _callee30$(_context30) {
      while (1) switch (_context30.prev = _context30.next) {
        case 0:
          marketplaceParams = _ref57.marketplaceParams;
          if (!marketplaceParams) {
            _context30.next = 5;
            break;
          }
          _context30.next = 4;
          return this.MarketplaceInfo({
            marketplaceParams: marketplaceParams
          });
        case 4:
          tenantId = _context30.sent.tenantId;
        case 5:
          _context30.t0 = Utils;
          _context30.next = 8;
          return this.client.authClient.MakeAuthServiceRequest({
            path: UrlJoin("as", "mkt", "names"),
            method: "GET",
            queryParams: tenantId ? {
              filter: "tenant:eq:".concat(tenantId)
            } : {}
          });
        case 8:
          _context30.t1 = _context30.sent;
          _context30.next = 11;
          return _context30.t0.ResponseToJson.call(_context30.t0, _context30.t1);
        case 11:
          return _context30.abrupt("return", _context30.sent);
        case 12:
        case "end":
          return _context30.stop();
      }
    }, _callee30, this);
  }));
  return function (_x17) {
    return _ref58.apply(this, arguments);
  };
}();

/**
 * Retrieve all valid edition names of the specified item. Full item edition names are required for filtering listing results by edition.
 *
 * @methodGroup Listings
 * @namedParams
 * @param {string} displayName - Display name of the item from which to request edition names
 *
 * @returns {Promise<Array<String>>} - A list of item editions
 */
exports.ListingEditionNames = /*#__PURE__*/function () {
  var _ref60 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee31(_ref59) {
    var displayName;
    return _regeneratorRuntime.wrap(function _callee31$(_context31) {
      while (1) switch (_context31.prev = _context31.next) {
        case 0:
          displayName = _ref59.displayName;
          _context31.t0 = Utils;
          _context31.next = 4;
          return this.client.authClient.MakeAuthServiceRequest({
            path: UrlJoin("as", "mkt", "editions"),
            queryParams: {
              filter: "nft/display_name:eq:".concat(displayName)
            },
            method: "GET"
          });
        case 4:
          _context31.t1 = _context31.sent;
          _context31.next = 7;
          return _context31.t0.ResponseToJson.call(_context31.t0, _context31.t1);
        case 7:
          return _context31.abrupt("return", _context31.sent);
        case 8:
        case "end":
          return _context31.stop();
      }
    }, _callee31, this);
  }));
  return function (_x18) {
    return _ref60.apply(this, arguments);
  };
}();

/**
 * Retrieve names of all valid attributes for listed items. Full attribute names and values are required for filtering listing results by attributes.
 *
 * Specify marketplace information to filter the results to only items offered in that marketplace.
 *
 * @methodGroup Listings
 * @namedParams
 * @param {Object=} marketplaceParams - Parameters of a marketplace to filter results by
 * @param {string=} displayName - Display name of the item from which to request attributes
 *
 * @returns {Promise<Array<String>>} - A list of valid attributes
 */
exports.ListingAttributes = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee32() {
  var _ref62,
    marketplaceParams,
    displayName,
    filters,
    attributes,
    _args32 = arguments;
  return _regeneratorRuntime.wrap(function _callee32$(_context32) {
    while (1) switch (_context32.prev = _context32.next) {
      case 0:
        _ref62 = _args32.length > 0 && _args32[0] !== undefined ? _args32[0] : {}, marketplaceParams = _ref62.marketplaceParams, displayName = _ref62.displayName;
        filters = [];
        if (!marketplaceParams) {
          _context32.next = 10;
          break;
        }
        _context32.t0 = filters;
        _context32.t1 = "tenant:eq:";
        _context32.next = 7;
        return this.MarketplaceInfo({
          marketplaceParams: marketplaceParams
        });
      case 7:
        _context32.t2 = _context32.sent.tenantId;
        _context32.t3 = _context32.t1.concat.call(_context32.t1, _context32.t2);
        _context32.t0.push.call(_context32.t0, _context32.t3);
      case 10:
        if (displayName) {
          filters.push("nft/display_name:eq:".concat(displayName));
        }
        _context32.t4 = Utils;
        _context32.next = 14;
        return this.client.authClient.MakeAuthServiceRequest({
          path: UrlJoin("as", "mkt", "attributes"),
          method: "GET",
          queryParams: {
            filter: filters
          }
        });
      case 14:
        _context32.t5 = _context32.sent;
        _context32.next = 17;
        return _context32.t4.ResponseToJson.call(_context32.t4, _context32.t5);
      case 17:
        attributes = _context32.sent;
        return _context32.abrupt("return", attributes.map(function (_ref63) {
          var trait_type = _ref63.trait_type,
            values = _ref63.values;
          return {
            name: trait_type,
            values: values
          };
        }).filter(function (_ref64) {
          var name = _ref64.name;
          return !["Content Fabric Hash", "Total Minted Supply", "Creator"].includes(name);
        }));
      case 19:
      case "end":
        return _context32.stop();
    }
  }, _callee32, this);
}));

/* PURCHASE / CLAIM */

/**
 * Claim the specified gift code
 *
 * Use the <a href="#.GiftClaimStatus">GiftClaimStatus</a> method to check minting status after claiming
 *
 * @methodGroup Purchase
 * @namedParams
 * @param {string} code - The OTP gift code to claim
 *
 * @returns {Promise<Object>} - Information about the claim, including the tenant associated with the item, the gift ID and the status op key
 */
exports.ClaimGift = /*#__PURE__*/function () {
  var _ref66 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee33(_ref65) {
    var code;
    return _regeneratorRuntime.wrap(function _callee33$(_context33) {
      while (1) switch (_context33.prev = _context33.next) {
        case 0:
          code = _ref65.code;
          _context33.next = 3;
          return Utils.ResponseToJson(this.client.authClient.MakeAuthServiceRequest({
            method: "POST",
            path: UrlJoin("as", "wlt", "gifts", "claim"),
            body: {
              otp_code: code
            },
            headers: {
              Authorization: "Bearer ".concat(this.AuthToken())
            }
          }));
        case 3:
          return _context33.abrupt("return", _context33.sent);
        case 4:
        case "end":
          return _context33.stop();
      }
    }, _callee33, this);
  }));
  return function (_x19) {
    return _ref66.apply(this, arguments);
  };
}();

/**
 * Claim the specified item from the specified marketplace
 *
 * Use the <a href="#.ClaimStatus">ClaimStatus</a> method to check minting status after claiming
 *
 * @methodGroup Purchase
 * @namedParams
 * @param {Object} marketplaceParams - Parameters of the marketplace
 * @param {string} sku - The SKU of the item to claim
 * @param {string=} email - Email address of the user. If specified, this will bind the user to the tenant of the specified marketplace
 */
exports.ClaimItem = /*#__PURE__*/function () {
  var _ref68 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee34(_ref67) {
    var marketplaceParams, sku, email, marketplaceInfo;
    return _regeneratorRuntime.wrap(function _callee34$(_context34) {
      while (1) switch (_context34.prev = _context34.next) {
        case 0:
          marketplaceParams = _ref67.marketplaceParams, sku = _ref67.sku, email = _ref67.email;
          _context34.next = 3;
          return this.MarketplaceInfo({
            marketplaceParams: marketplaceParams
          });
        case 3:
          marketplaceInfo = _context34.sent;
          _context34.next = 6;
          return this.client.authClient.MakeAuthServiceRequest({
            method: "POST",
            path: UrlJoin("as", "wlt", "act", marketplaceInfo.tenant_id),
            body: {
              op: "nft-claim",
              sid: marketplaceInfo.marketplaceId,
              sku: sku,
              email: email
            },
            headers: {
              Authorization: "Bearer ".concat(this.AuthToken())
            }
          });
        case 6:
        case "end":
          return _context34.stop();
      }
    }, _callee34, this);
  }));
  return function (_x20) {
    return _ref68.apply(this, arguments);
  };
}();

/**
 * Redirect to the wallet app to purchase the specified item from the specified marketplace
 *
 * Use the <a href="#.PurchaseStatus">PurchaseStatus</a> method to check minting status after purchasing
 *
 * @methodGroup Purchase
 * @namedParams
 * @param {Object} marketplaceParams - Parameters of the marketplace
 * @param {string} sku - The SKU of the item to claim
 * @param {string=} confirmationId - Confirmation ID with which to reference this purchase. If not specified, a confirmation ID will be automatically generated. On success, the user will be returned to `successUrl` with the `confirmationId` as a URL parameter.
 * @param {string} successUrl - The URL to redirect back to upon successful purchase
 * @param {string} cancelUrl - The URL to redirect back to upon cancellation of purchase
 */
exports.PurchaseItem = /*#__PURE__*/function () {
  var _ref70 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee35(_ref69) {
    var marketplaceParams, sku, confirmationId, successUrl, cancelUrl, marketplaceInfo;
    return _regeneratorRuntime.wrap(function _callee35$(_context35) {
      while (1) switch (_context35.prev = _context35.next) {
        case 0:
          marketplaceParams = _ref69.marketplaceParams, sku = _ref69.sku, confirmationId = _ref69.confirmationId, successUrl = _ref69.successUrl, cancelUrl = _ref69.cancelUrl;
          _context35.next = 3;
          return this.MarketplaceInfo({
            marketplaceParams: marketplaceParams
          });
        case 3:
          marketplaceInfo = _context35.sent;
          window.location.href = this.FlowURL({
            type: "action",
            flow: "purchase",
            marketplaceId: marketplaceInfo.marketplaceId,
            parameters: {
              sku: sku,
              confirmationId: confirmationId,
              successUrl: successUrl,
              cancelUrl: cancelUrl,
              login: true,
              auth: this.ClientAuthToken()
            }
          });
        case 5:
        case "end":
          return _context35.stop();
      }
    }, _callee35, this);
  }));
  return function (_x21) {
    return _ref70.apply(this, arguments);
  };
}();

/**
 * Redirect to the wallet app to purchase the specified listing
 *
 * Use the <a href="#.PurchaseStatus">ListingPurchaseStatus</a> method to check minting status after purchasing
 *
 * @methodGroup Purchase
 * @namedParams
 * @param {Object=} marketplaceParams - Parameters of the marketplace
 * @param {string} listingId - The SKU of the item to claim
 * @param {string=} confirmationId - Confirmation ID with which to reference this purchase. If not specified, a confirmation ID will be automatically generated. On success, the user will be returned to `successUrl` with the `confirmationId` as a URL parameter.
 * @param {string} successUrl - The URL to redirect back to upon successful purchase
 * @param {string} cancelUrl - The URL to redirect back to upon cancellation of purchase
 */
exports.PurchaseListing = /*#__PURE__*/function () {
  var _ref72 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee36(_ref71) {
    var marketplaceParams, listingId, confirmationId, successUrl, cancelUrl, marketplaceInfo;
    return _regeneratorRuntime.wrap(function _callee36$(_context36) {
      while (1) switch (_context36.prev = _context36.next) {
        case 0:
          marketplaceParams = _ref71.marketplaceParams, listingId = _ref71.listingId, confirmationId = _ref71.confirmationId, successUrl = _ref71.successUrl, cancelUrl = _ref71.cancelUrl;
          if (!marketplaceParams) {
            _context36.next = 5;
            break;
          }
          _context36.next = 4;
          return this.MarketplaceInfo({
            marketplaceParams: marketplaceParams
          });
        case 4:
          marketplaceInfo = _context36.sent;
        case 5:
          window.location.href = this.FlowURL({
            type: "action",
            flow: "purchase",
            marketplaceId: marketplaceInfo && marketplaceInfo.marketplaceId,
            parameters: {
              listingId: listingId,
              confirmationId: confirmationId,
              successUrl: successUrl,
              cancelUrl: cancelUrl,
              login: true,
              auth: this.ClientAuthToken()
            }
          });
        case 6:
        case "end":
          return _context36.stop();
      }
    }, _callee36, this);
  }));
  return function (_x22) {
    return _ref72.apply(this, arguments);
  };
}();

/* MINTING STATUS */

/**
 * Return status of the specified listing purchase
 *
 * @methodGroup Status
 * @namedParams
 * @param {string} listingId - The ID of the listing
 * @param {string} confirmationId - The confirmation ID of the purchase
 *
 * @returns {Promise<Object>} - The status of the purchase
 */
exports.ListingPurchaseStatus = /*#__PURE__*/function () {
  var _ref74 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee37(_ref73) {
    var listingId, confirmationId, listingStatus, statuses;
    return _regeneratorRuntime.wrap(function _callee37$(_context37) {
      while (1) switch (_context37.prev = _context37.next) {
        case 0:
          listingId = _ref73.listingId, confirmationId = _ref73.confirmationId;
          _context37.prev = 1;
          _context37.next = 4;
          return this.ListingStatus({
            listingId: listingId
          });
        case 4:
          listingStatus = _context37.sent;
          if (listingStatus) {
            _context37.next = 7;
            break;
          }
          throw Error("Unable to find info for listing " + listingId);
        case 7:
          _context37.next = 9;
          return this.MintingStatus({
            tenantId: listingStatus.tenant
          });
        case 9:
          statuses = _context37.sent;
          return _context37.abrupt("return", statuses.find(function (status) {
            return status.op === "nft-transfer" && status.extra && status.extra[0] === confirmationId;
          }) || {
            status: "none"
          });
        case 13:
          _context37.prev = 13;
          _context37.t0 = _context37["catch"](1);
          this.Log(_context37.t0, true);
          return _context37.abrupt("return", {
            status: "unknown"
          });
        case 17:
        case "end":
          return _context37.stop();
      }
    }, _callee37, this, [[1, 13]]);
  }));
  return function (_x23) {
    return _ref74.apply(this, arguments);
  };
}();

/**
 * Return status of the specified marketplace purchase
 *
 * @methodGroup Status
 * @namedParams
 * @param {Object} marketplaceParams - Parameters of the marketplace
 * @param {string} confirmationId - The confirmation ID of the purchase
 *
 * @returns {Promise<Object>} - The minting status of the purchaseed item(s)
 */
exports.PurchaseStatus = /*#__PURE__*/function () {
  var _ref76 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee38(_ref75) {
    var marketplaceParams, confirmationId, marketplaceInfo, statuses;
    return _regeneratorRuntime.wrap(function _callee38$(_context38) {
      while (1) switch (_context38.prev = _context38.next) {
        case 0:
          marketplaceParams = _ref75.marketplaceParams, confirmationId = _ref75.confirmationId;
          _context38.prev = 1;
          _context38.next = 4;
          return this.MarketplaceInfo({
            marketplaceParams: marketplaceParams
          });
        case 4:
          marketplaceInfo = _context38.sent;
          _context38.next = 7;
          return this.MintingStatus({
            tenantId: marketplaceInfo.tenantId
          });
        case 7:
          statuses = _context38.sent;
          return _context38.abrupt("return", statuses.find(function (status) {
            return status.op === "nft-buy" && status.confirmationId === confirmationId;
          }) || {
            status: "none"
          });
        case 11:
          _context38.prev = 11;
          _context38.t0 = _context38["catch"](1);
          this.Log(_context38.t0, true);
          return _context38.abrupt("return", {
            status: "unknown"
          });
        case 15:
        case "end":
          return _context38.stop();
      }
    }, _callee38, this, [[1, 11]]);
  }));
  return function (_x24) {
    return _ref76.apply(this, arguments);
  };
}();

/**
 * Return status of the specified item claim
 *
 * @methodGroup Status
 * @namedParams
 * @param {Object} marketplaceParams - Parameters of the marketplace
 * @param {string} sku - The SKU of the item claimed
 *
 * @returns {Promise<Object>} - The minting status of the claim
 */
exports.ClaimStatus = /*#__PURE__*/function () {
  var _ref78 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee39(_ref77) {
    var marketplaceParams, sku, marketplaceInfo, statuses;
    return _regeneratorRuntime.wrap(function _callee39$(_context39) {
      while (1) switch (_context39.prev = _context39.next) {
        case 0:
          marketplaceParams = _ref77.marketplaceParams, sku = _ref77.sku;
          _context39.prev = 1;
          _context39.next = 4;
          return this.MarketplaceInfo({
            marketplaceParams: marketplaceParams
          });
        case 4:
          marketplaceInfo = _context39.sent;
          _context39.next = 7;
          return this.MintingStatus({
            tenantId: marketplaceInfo.tenantId
          });
        case 7:
          statuses = _context39.sent;
          return _context39.abrupt("return", statuses.find(function (status) {
            return status.op === "nft-claim" && status.marketplaceId === marketplaceInfo.marketplaceId && status.confirmationId === sku;
          }) || {
            status: "none"
          });
        case 11:
          _context39.prev = 11;
          _context39.t0 = _context39["catch"](1);
          this.Log(_context39.t0, true);
          return _context39.abrupt("return", {
            status: "unknown"
          });
        case 15:
        case "end":
          return _context39.stop();
      }
    }, _callee39, this, [[1, 11]]);
  }));
  return function (_x25) {
    return _ref78.apply(this, arguments);
  };
}();

/**
 * Return status of the specified gift claim
 *
 * @methodGroup Status
 * @namedParams
 * @param {Object} marketplaceParams - Parameters of the marketplace
 * @param {string=} confirmationId - The confirmation ID of the gift purchase
 * @param {string=} giftId - The ID of the claimed gift
 *
 * @returns {Promise<Object>} - The transfer status of the gift claim
 */
exports.GiftClaimStatus = /*#__PURE__*/function () {
  var _ref80 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee40(_ref79) {
    var marketplaceParams, confirmationId, giftId, marketplaceInfo, statuses, responses;
    return _regeneratorRuntime.wrap(function _callee40$(_context40) {
      while (1) switch (_context40.prev = _context40.next) {
        case 0:
          marketplaceParams = _ref79.marketplaceParams, confirmationId = _ref79.confirmationId, giftId = _ref79.giftId;
          _context40.prev = 1;
          _context40.next = 4;
          return this.MarketplaceInfo({
            marketplaceParams: marketplaceParams
          });
        case 4:
          marketplaceInfo = _context40.sent;
          _context40.next = 7;
          return this.MintingStatus({
            tenantId: marketplaceInfo.tenantId
          });
        case 7:
          statuses = _context40.sent;
          // Status is a list of transfer statuses, may be multiple if quantity > 1
          responses = statuses.filter(function (status) {
            return status.op === "nft-transfer" && (confirmationId && status.confirmationId === confirmationId || giftId && status.giftId === giftId);
          }) || {
            status: "none"
          };
          if (!(responses.length === 0)) {
            _context40.next = 13;
            break;
          }
          return _context40.abrupt("return", {
            status: "none"
          });
        case 13:
          if (!responses.find(function (response) {
            return response.status === "failed";
          })) {
            _context40.next = 17;
            break;
          }
          return _context40.abrupt("return", {
            status: "failed",
            op: "nft-transfer",
            transfer_statuses: responses
          });
        case 17:
          if (!responses.find(function (response) {
            return response.status !== "complete";
          })) {
            _context40.next = 21;
            break;
          }
          return _context40.abrupt("return", {
            status: "pending",
            op: "nft-transfer",
            transfer_statuses: responses
          });
        case 21:
          return _context40.abrupt("return", {
            status: "complete",
            op: "nft-transfer",
            transfer_statuses: responses,
            items: responses.map(function (response) {
              return {
                token_addr: response.address,
                token_id_str: response.tokenId
              };
            })
          });
        case 22:
          _context40.next = 28;
          break;
        case 24:
          _context40.prev = 24;
          _context40.t0 = _context40["catch"](1);
          this.Log(_context40.t0, true);
          return _context40.abrupt("return", {
            status: "unknown"
          });
        case 28:
        case "end":
          return _context40.stop();
      }
    }, _callee40, this, [[1, 24]]);
  }));
  return function (_x26) {
    return _ref80.apply(this, arguments);
  };
}();

/**
 * Return status of the specified entitlement claim
 *
 * @methodGroup Status
 * @namedParams
 * @param {Object} marketplaceParams - Parameters of the marketplace
 * @param {string} purchaseId - The purchase ID of the entitlement, for confirmation of status
 *
 * @returns {Promise<Object>} - The mint status of the entitlement claim
 */
exports.EntitlementClaimStatus = /*#__PURE__*/function () {
  var _ref82 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee41(_ref81) {
    var marketplaceParams, purchaseId, marketplaceInfo, statuses, responses;
    return _regeneratorRuntime.wrap(function _callee41$(_context41) {
      while (1) switch (_context41.prev = _context41.next) {
        case 0:
          marketplaceParams = _ref81.marketplaceParams, purchaseId = _ref81.purchaseId;
          _context41.prev = 1;
          _context41.next = 4;
          return this.MarketplaceInfo({
            marketplaceParams: marketplaceParams
          });
        case 4:
          marketplaceInfo = _context41.sent;
          _context41.next = 7;
          return this.MintingStatus({
            tenantId: marketplaceInfo.tenantId
          });
        case 7:
          statuses = _context41.sent;
          responses = statuses.filter(function (status) {
            return status.op === "nft-claim-entitlement" && purchaseId && purchaseId == status.confirmationId;
          }) || {
            status: "none"
          };
          if (!(responses.length === 0)) {
            _context41.next = 13;
            break;
          }
          return _context41.abrupt("return", {
            status: "none"
          });
        case 13:
          if (!responses.find(function (response) {
            return response.status === "complete";
          })) {
            _context41.next = 17;
            break;
          }
          return _context41.abrupt("return", {
            status: "complete",
            op: "nft-claim-entitlement",
            items: [{
              token_addr: responses[0].address,
              token_id: responses[0].tokenId
            }]
          });
        case 17:
          if (!responses.find(function (response) {
            return response.status === "error";
          })) {
            _context41.next = 21;
            break;
          }
          return _context41.abrupt("return", {
            status: "error",
            op: "nft-claim-entitlement"
          });
        case 21:
          return _context41.abrupt("return", {
            status: "pending",
            op: "nft-claim-entitlement"
          });
        case 22:
          _context41.next = 28;
          break;
        case 24:
          _context41.prev = 24;
          _context41.t0 = _context41["catch"](1);
          this.Log(_context41.t0, true);
          return _context41.abrupt("return", {
            status: "unknown"
          });
        case 28:
        case "end":
          return _context41.stop();
      }
    }, _callee41, this, [[1, 24]]);
  }));
  return function (_x27) {
    return _ref82.apply(this, arguments);
  };
}();

/**
 * Return status of the specified pack opening
 *
 * @methodGroup Status
 * @namedParams
 * @param {string} contractAddress - The NFT contract address of the opened pack
 * @param {string} tokenId - The token ID of the opened pack
 *
 * @returns {Promise<Object>} - The status of the pack opening
 */
exports.PackOpenStatus = /*#__PURE__*/function () {
  var _ref84 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee42(_ref83) {
    var contractAddress, tokenId, tenantConfig, statuses;
    return _regeneratorRuntime.wrap(function _callee42$(_context42) {
      while (1) switch (_context42.prev = _context42.next) {
        case 0:
          contractAddress = _ref83.contractAddress, tokenId = _ref83.tokenId;
          _context42.prev = 1;
          _context42.next = 4;
          return this.TenantConfiguration({
            contractAddress: contractAddress
          });
        case 4:
          tenantConfig = _context42.sent;
          _context42.next = 7;
          return this.MintingStatus({
            tenantId: tenantConfig.tenant
          });
        case 7:
          statuses = _context42.sent;
          return _context42.abrupt("return", statuses.find(function (status) {
            return status.op === "nft-open" && Utils.EqualAddress(contractAddress, status.address) && status.tokenId === tokenId;
          }) || {
            status: "none"
          });
        case 11:
          _context42.prev = 11;
          _context42.t0 = _context42["catch"](1);
          this.Log(_context42.t0, true);
          return _context42.abrupt("return", {
            status: "unknown"
          });
        case 15:
        case "end":
          return _context42.stop();
      }
    }, _callee42, this, [[1, 11]]);
  }));
  return function (_x28) {
    return _ref84.apply(this, arguments);
  };
}();

/**
 * Return status of the specified collection redemption
 *
 * @methodGroup Status
 * @namedParams
 * @param {Object} marketplaceParams - Parameters of the marketplace
 * @param {string} confirmationId - The confirmation ID of the redemption
 *
 * @returns {Promise<Object>} - The status of the collection redemption
 */
exports.CollectionRedemptionStatus = /*#__PURE__*/function () {
  var _ref86 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee43(_ref85) {
    var marketplaceParams, confirmationId, statuses;
    return _regeneratorRuntime.wrap(function _callee43$(_context43) {
      while (1) switch (_context43.prev = _context43.next) {
        case 0:
          marketplaceParams = _ref85.marketplaceParams, confirmationId = _ref85.confirmationId;
          _context43.prev = 1;
          _context43.next = 4;
          return this.MintingStatus({
            marketplaceParams: marketplaceParams
          });
        case 4:
          statuses = _context43.sent;
          return _context43.abrupt("return", statuses.find(function (status) {
            return status.op === "nft-redeem" && status.confirmationId === confirmationId;
          }) || {
            status: "none"
          });
        case 8:
          _context43.prev = 8;
          _context43.t0 = _context43["catch"](1);
          this.Log(_context43.t0, true);
          return _context43.abrupt("return", {
            status: "unknown"
          });
        case 12:
        case "end":
          return _context43.stop();
      }
    }, _callee43, this, [[1, 8]]);
  }));
  return function (_x29) {
    return _ref86.apply(this, arguments);
  };
}();

/**
 * Return status of the specified redeemable offer
 *
 * @methodGroup Status
 * @namedParams
 * @param {string=} tenantId - ID of the tenant for this NFT (not required if `marketplaceParams` is specified)
 * @param {Object=} marketplaceParams - Parameters of the marketplace for this NFT (not required if `tenantId` is specified)
 * @param {string} contractAddress - The address of the NFT contract
 * @param {string} tokenId - The token ID of the NFT
 * @param {string} offerId - The ID of the offer
 *
 * @returns {Promise<Object>} - The status of the offer redemption
 */
exports.RedeemableOfferStatus = /*#__PURE__*/function () {
  var _ref88 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee44(_ref87) {
    var tenantId, marketplaceParams, contractAddress, tokenId, offerId, statuses;
    return _regeneratorRuntime.wrap(function _callee44$(_context44) {
      while (1) switch (_context44.prev = _context44.next) {
        case 0:
          tenantId = _ref87.tenantId, marketplaceParams = _ref87.marketplaceParams, contractAddress = _ref87.contractAddress, tokenId = _ref87.tokenId, offerId = _ref87.offerId;
          _context44.prev = 1;
          _context44.next = 4;
          return this.MintingStatus({
            marketplaceParams: marketplaceParams,
            tenantId: tenantId
          });
        case 4:
          statuses = _context44.sent;
          contractAddress = Utils.FormatAddress(contractAddress);
          return _context44.abrupt("return", statuses.find(function (status) {
            return status.op === "nft-offer-redeem" && Utils.EqualAddress(status.address, contractAddress) && status.tokenId === (tokenId || "").toString() && status.offerId === (offerId || "").toString();
          }) || {
            status: "none"
          });
        case 9:
          _context44.prev = 9;
          _context44.t0 = _context44["catch"](1);
          this.Log(_context44.t0, true);
          return _context44.abrupt("return", {
            status: "unknown"
          });
        case 13:
        case "end":
          return _context44.stop();
      }
    }, _callee44, this, [[1, 9]]);
  }));
  return function (_x30) {
    return _ref88.apply(this, arguments);
  };
}();
exports.RedeemableCustomFulfillmentInfo = /*#__PURE__*/function () {
  var _ref90 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee45(_ref89) {
    var redeemableTransactionId;
    return _regeneratorRuntime.wrap(function _callee45$(_context45) {
      while (1) switch (_context45.prev = _context45.next) {
        case 0:
          redeemableTransactionId = _ref89.redeemableTransactionId;
          _context45.next = 3;
          return Utils.ResponseToJson(this.stateStoreClient.Request({
            method: "GET",
            path: UrlJoin("code-fulfillment", this.network === "main" ? "main" : "demov3", "fulfill", redeemableTransactionId),
            headers: {
              Authorization: "Bearer ".concat(this.AuthToken())
            }
          }));
        case 3:
          return _context45.abrupt("return", _context45.sent);
        case 4:
        case "end":
          return _context45.stop();
      }
    }, _callee45, this);
  }));
  return function (_x31) {
    return _ref90.apply(this, arguments);
  };
}();

/* EVENTS */

exports.LoadDrop = /*#__PURE__*/function () {
  var _ref92 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee46(_ref91) {
    var _this = this;
    var tenantSlug, eventSlug, dropId, event, eventId;
    return _regeneratorRuntime.wrap(function _callee46$(_context46) {
      while (1) switch (_context46.prev = _context46.next) {
        case 0:
          tenantSlug = _ref91.tenantSlug, eventSlug = _ref91.eventSlug, dropId = _ref91.dropId;
          if (!this.drops) {
            this.drops = {};
          }
          if (!this.drops[tenantSlug]) {
            this.drops[tenantSlug] = {};
          }
          if (!this.drops[tenantSlug][eventSlug]) {
            this.drops[tenantSlug][eventSlug] = {};
          }
          if (this.drops[tenantSlug][eventSlug][dropId]) {
            _context46.next = 13;
            break;
          }
          _context46.next = 7;
          return this.client.ContentObjectMetadata({
            libraryId: this.mainSiteLibraryId,
            objectId: this.mainSiteId,
            metadataSubtree: UrlJoin("public", "asset_metadata", "tenants", tenantSlug, "sites", eventSlug, "info"),
            resolveLinks: true,
            linkDepthLimit: 2,
            resolveIncludeSource: true,
            produceLinkUrls: true,
            select: [".", "drops"],
            noAuth: true
          });
        case 7:
          _context46.t0 = _context46.sent;
          if (_context46.t0) {
            _context46.next = 10;
            break;
          }
          _context46.t0 = [];
        case 10:
          event = _context46.t0;
          eventId = Utils.DecodeVersionHash(event["."].source).objectId;
          event.drops.forEach(function (drop) {
            drop = _objectSpread(_objectSpread({}, drop), {}, {
              eventId: eventId
            });
            _this.drops[tenantSlug][eventSlug][drop.uuid] = drop;
            _this.drops[drop.uuid] = drop;
          });
        case 13:
          return _context46.abrupt("return", this.drops[dropId]);
        case 14:
        case "end":
          return _context46.stop();
      }
    }, _callee46, this);
  }));
  return function (_x32) {
    return _ref92.apply(this, arguments);
  };
}();
exports.SubmitDropVote = /*#__PURE__*/function () {
  var _ref94 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee47(_ref93) {
    var marketplaceParams, eventId, dropId, sku, marketplaceInfo;
    return _regeneratorRuntime.wrap(function _callee47$(_context47) {
      while (1) switch (_context47.prev = _context47.next) {
        case 0:
          marketplaceParams = _ref93.marketplaceParams, eventId = _ref93.eventId, dropId = _ref93.dropId, sku = _ref93.sku;
          _context47.next = 3;
          return this.MarketplaceInfo({
            marketplaceParams: marketplaceParams
          });
        case 3:
          marketplaceInfo = _context47.sent;
          _context47.next = 6;
          return this.client.authClient.MakeAuthServiceRequest({
            path: UrlJoin("as", "wlt", "act", marketplaceInfo.tenant_id),
            method: "POST",
            body: {
              op: "vote-drop",
              evt: eventId,
              id: dropId,
              itm: sku
            },
            headers: {
              Authorization: "Bearer ".concat(this.AuthToken())
            }
          });
        case 6:
        case "end":
          return _context47.stop();
      }
    }, _callee47, this);
  }));
  return function (_x33) {
    return _ref94.apply(this, arguments);
  };
}();
exports.DropStatus = /*#__PURE__*/function () {
  var _ref96 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee48(_ref95) {
    var marketplace, eventId, dropId, response;
    return _regeneratorRuntime.wrap(function _callee48$(_context48) {
      while (1) switch (_context48.prev = _context48.next) {
        case 0:
          marketplace = _ref95.marketplace, eventId = _ref95.eventId, dropId = _ref95.dropId;
          _context48.prev = 1;
          _context48.next = 4;
          return Utils.ResponseToJson(this.client.authClient.MakeAuthServiceRequest({
            path: UrlJoin("as", "wlt", "act", marketplace.tenant_id, eventId, dropId),
            method: "GET",
            headers: {
              Authorization: "Bearer ".concat(this.AuthToken())
            }
          }));
        case 4:
          response = _context48.sent;
          return _context48.abrupt("return", response.sort(function (a, b) {
            return a.ts > b.ts ? 1 : -1;
          })[0] || {
            status: "none"
          });
        case 8:
          _context48.prev = 8;
          _context48.t0 = _context48["catch"](1);
          this.Log(_context48.t0, true);
          return _context48.abrupt("return", "");
        case 12:
        case "end":
          return _context48.stop();
      }
    }, _callee48, this, [[1, 8]]);
  }));
  return function (_x34) {
    return _ref96.apply(this, arguments);
  };
}();

/* OFFERS */

/**
 * Retrieve offers for the specified parameters
 *
 * @methodGroup Offers
 * @namedParams
 * @param {string=} contractAddress - The address of an NFT contract
 * @param {string=} tokenId - The token ID of an NFT
 * @param {string=} buyerAddress - The address of the offerrer
 * @param {string=} sellerAddress - The address of the offerree
 * @param {Array<String>=} statuses - Status to filter results by. Allowed values: "ACTIVE", "ACCEPTED", "CANCELLED", "DECLINED", "INVALID"
 * @param {number} start=0 - The index to start from
 * @param {number=} limit=10 - The maximum number of results to return
 *
 * @returns {Promise<Array<Object>>} - Offers matching the specified filters
 */
exports.MarketplaceOffers = /*#__PURE__*/function () {
  var _ref98 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee49(_ref97) {
    var contractAddress, tokenId, buyerAddress, sellerAddress, statuses, _ref97$start, start, _ref97$limit, limit, path, queryParams, offers;
    return _regeneratorRuntime.wrap(function _callee49$(_context49) {
      while (1) switch (_context49.prev = _context49.next) {
        case 0:
          contractAddress = _ref97.contractAddress, tokenId = _ref97.tokenId, buyerAddress = _ref97.buyerAddress, sellerAddress = _ref97.sellerAddress, statuses = _ref97.statuses, _ref97$start = _ref97.start, start = _ref97$start === void 0 ? 0 : _ref97$start, _ref97$limit = _ref97.limit, limit = _ref97$limit === void 0 ? 10 : _ref97$limit;
          path = UrlJoin("as", "mkt", "offers", "ls");
          if (buyerAddress) {
            path = UrlJoin(path, "b", Utils.FormatAddress(buyerAddress));
          } else if (sellerAddress) {
            path = UrlJoin(path, "s", Utils.FormatAddress(sellerAddress));
          }
          if (contractAddress) {
            path = UrlJoin(path, "c", Utils.FormatAddress(contractAddress));
            if (tokenId) {
              path = UrlJoin(path, "t", tokenId);
            }
          }
          queryParams = {
            start: start,
            limit: limit
          };
          if (statuses && statuses.length > 0) {
            queryParams.include = statuses.join(",");
          }
          _context49.next = 8;
          return Utils.ResponseToJson(this.client.authClient.MakeAuthServiceRequest({
            path: path,
            method: "GET",
            queryParams: queryParams
          }));
        case 8:
          offers = _context49.sent;
          return _context49.abrupt("return", offers.map(function (offer) {
            return _objectSpread(_objectSpread({}, offer), {}, {
              created: offer.created * 1000,
              updated: offer.updated * 1000,
              expiration: offer.expiration * 1000
            });
          }));
        case 10:
        case "end":
          return _context49.stop();
      }
    }, _callee49, this);
  }));
  return function (_x35) {
    return _ref98.apply(this, arguments);
  };
}();

/**
 * <b><i>Requires login</i></b>
 *
 * Create or update an offer on the specified NFT
 *
 * @methodGroup Offers
 * @namedParams
 * @param {string} contractAddress - The contract address of the NFT
 * @param {string} tokenId - The token ID of the NFT
 * @param {string=} offerId - IF modifying an existing offer, the ID of the offer
 * @param {number} price - The amount to offer
 * @param {number=} expiresAt - The time (in epoch ms) the offer will expire
 *
 * @returns {Promise<Object>} - Info about the created/updated offer
 */
exports.CreateMarketplaceOffer = /*#__PURE__*/function () {
  var _ref100 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee50(_ref99) {
    var contractAddress, tokenId, offerId, price, expiresAt, response;
    return _regeneratorRuntime.wrap(function _callee50$(_context50) {
      while (1) switch (_context50.prev = _context50.next) {
        case 0:
          contractAddress = _ref99.contractAddress, tokenId = _ref99.tokenId, offerId = _ref99.offerId, price = _ref99.price, expiresAt = _ref99.expiresAt;
          if (!offerId) {
            _context50.next = 7;
            break;
          }
          _context50.next = 4;
          return Utils.ResponseToJson(this.client.authClient.MakeAuthServiceRequest({
            path: UrlJoin("as", "wlt", "mkt", "offers", offerId),
            method: "PUT",
            body: {
              price: price,
              expiration: Math.floor(expiresAt / 1000)
            },
            headers: {
              Authorization: "Bearer ".concat(this.AuthToken())
            }
          }));
        case 4:
          response = _context50.sent;
          _context50.next = 10;
          break;
        case 7:
          _context50.next = 9;
          return Utils.ResponseToJson(this.client.authClient.MakeAuthServiceRequest({
            path: UrlJoin("as", "wlt", "mkt", "offers", contractAddress, tokenId),
            method: "POST",
            body: {
              contract: contractAddress,
              token: tokenId,
              price: price,
              expiration: Math.floor(expiresAt / 1000)
            },
            headers: {
              Authorization: "Bearer ".concat(this.AuthToken())
            }
          }));
        case 9:
          response = _context50.sent;
        case 10:
          return _context50.abrupt("return", response.offer_id);
        case 11:
        case "end":
          return _context50.stop();
      }
    }, _callee50, this);
  }));
  return function (_x36) {
    return _ref100.apply(this, arguments);
  };
}();

/**
 * <b><i>Requires login</i></b>
 *
 * Cancel the specified offer
 *
 * @methodGroup Offers
 * @namedParams
 * @param {string} offerId - The ID of the offer
 */
exports.RemoveMarketplaceOffer = /*#__PURE__*/function () {
  var _ref102 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee51(_ref101) {
    var offerId;
    return _regeneratorRuntime.wrap(function _callee51$(_context51) {
      while (1) switch (_context51.prev = _context51.next) {
        case 0:
          offerId = _ref101.offerId;
          _context51.next = 3;
          return this.client.authClient.MakeAuthServiceRequest({
            path: UrlJoin("as", "wlt", "mkt", "offers", offerId),
            method: "DELETE",
            headers: {
              Authorization: "Bearer ".concat(this.AuthToken())
            }
          });
        case 3:
          return _context51.abrupt("return", _context51.sent);
        case 4:
        case "end":
          return _context51.stop();
      }
    }, _callee51, this);
  }));
  return function (_x37) {
    return _ref102.apply(this, arguments);
  };
}();

/**
 * <b><i>Requires login</i></b>
 *
 * Accept the specified offer
 *
 * @methodGroup Offers
 * @namedParams
 * @param {string} offerId - The ID of the offer
 */
exports.AcceptMarketplaceOffer = /*#__PURE__*/function () {
  var _ref104 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee52(_ref103) {
    var offerId;
    return _regeneratorRuntime.wrap(function _callee52$(_context52) {
      while (1) switch (_context52.prev = _context52.next) {
        case 0:
          offerId = _ref103.offerId;
          _context52.next = 3;
          return this.client.authClient.MakeAuthServiceRequest({
            path: UrlJoin("as", "wlt", "mkt", "offers", "accept", offerId),
            method: "PUT",
            headers: {
              Authorization: "Bearer ".concat(this.AuthToken())
            }
          });
        case 3:
          return _context52.abrupt("return", _context52.sent);
        case 4:
        case "end":
          return _context52.stop();
      }
    }, _callee52, this);
  }));
  return function (_x38) {
    return _ref104.apply(this, arguments);
  };
}();

/**
 * <b><i>Requires login</i></b>
 *
 * Reject the specified offer
 *
 * @methodGroup Offers
 * @namedParams
 * @param {string} offerId - The ID of the offer
 */
exports.RejectMarketplaceOffer = /*#__PURE__*/function () {
  var _ref106 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee53(_ref105) {
    var offerId;
    return _regeneratorRuntime.wrap(function _callee53$(_context53) {
      while (1) switch (_context53.prev = _context53.next) {
        case 0:
          offerId = _ref105.offerId;
          _context53.next = 3;
          return this.client.authClient.MakeAuthServiceRequest({
            path: UrlJoin("as", "wlt", "mkt", "offers", "decline", offerId),
            method: "PUT",
            headers: {
              Authorization: "Bearer ".concat(this.AuthToken())
            }
          });
        case 3:
          return _context53.abrupt("return", _context53.sent);
        case 4:
        case "end":
          return _context53.stop();
      }
    }, _callee53, this);
  }));
  return function (_x39) {
    return _ref106.apply(this, arguments);
  };
}();

/* Voting */

/**
 * Retrieve the current status of the specified voting event
 *
 * @methodGroup Voting
 * @namedParams
 * @param {string} tenantId - The tenant ID of the marketplace in which the voting event is specified
 * @param {string} votingEventId - The ID of the voting event
 *
 * @returns {Promise<Object>} - Info about the voting event, including the current user's votes and the current total voting tally
 */
exports.VoteStatus = /*#__PURE__*/function () {
  var _ref108 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee54(_ref107) {
    var tenantId, votingEventId;
    return _regeneratorRuntime.wrap(function _callee54$(_context54) {
      while (1) switch (_context54.prev = _context54.next) {
        case 0:
          tenantId = _ref107.tenantId, votingEventId = _ref107.votingEventId;
          _context54.next = 3;
          return Utils.ResponseToJson(this.client.authClient.MakeAuthServiceRequest({
            path: UrlJoin("as", "votes", tenantId, votingEventId),
            headers: {
              Authorization: "Bearer ".concat(this.AuthToken())
            }
          }));
        case 3:
          return _context54.abrupt("return", _context54.sent);
        case 4:
        case "end":
          return _context54.stop();
      }
    }, _callee54, this);
  }));
  return function (_x40) {
    return _ref108.apply(this, arguments);
  };
}();

/**
 * <b><i>Requires login</i></b>
 *
 * Cast a vote for the specified item in the specified voting event
 *
 * @methodGroup Voting
 * @namedParams
 * @param {string} tenantId - The tenant ID of the marketplace in which the voting event is specified
 * @param {string} votingEventId - The ID of the voting event
 * @param {string} sku - The SKU of the item to vote for
 *
 * @returns {Promise<Object>} - Info about the voting event, including the current user's votes and the current total voting tally
 */
exports.CastVote = /*#__PURE__*/function () {
  var _ref110 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee55(_ref109) {
    var tenantId, votingEventId, sku;
    return _regeneratorRuntime.wrap(function _callee55$(_context55) {
      while (1) switch (_context55.prev = _context55.next) {
        case 0:
          tenantId = _ref109.tenantId, votingEventId = _ref109.votingEventId, sku = _ref109.sku;
          _context55.next = 3;
          return Utils.ResponseToJson(this.client.authClient.MakeAuthServiceRequest({
            path: UrlJoin("as", "votes", tenantId, votingEventId, sku),
            method: "POST",
            headers: {
              Authorization: "Bearer ".concat(this.AuthToken())
            }
          }));
        case 3:
          return _context55.abrupt("return", _context55.sent);
        case 4:
        case "end":
          return _context55.stop();
      }
    }, _callee55, this);
  }));
  return function (_x41) {
    return _ref110.apply(this, arguments);
  };
}();

/**
 * <b><i>Requires login</i></b>
 *
 * Revoke a previously cast vote for the specified item in the specified voting event
 *
 * @methodGroup Voting
 * @namedParams
 * @param {string} tenantId - The tenant ID of the marketplace in which the voting event is specified
 * @param {string} votingEventId - The ID of the voting event
 * @param {string} sku - The SKU of the item to vote for
 *
 * @returns {Promise<Object>} - Info about the voting event, including the current user's votes and the current total voting tally
 */
exports.RevokeVote = /*#__PURE__*/function () {
  var _ref112 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee56(_ref111) {
    var tenantId, votingEventId, sku;
    return _regeneratorRuntime.wrap(function _callee56$(_context56) {
      while (1) switch (_context56.prev = _context56.next) {
        case 0:
          tenantId = _ref111.tenantId, votingEventId = _ref111.votingEventId, sku = _ref111.sku;
          _context56.next = 3;
          return Utils.ResponseToJson(this.client.authClient.MakeAuthServiceRequest({
            path: UrlJoin("as", "votes", tenantId, votingEventId, sku),
            method: "DELETE",
            headers: {
              Authorization: "Bearer ".concat(this.AuthToken())
            }
          }));
        case 3:
          return _context56.abrupt("return", _context56.sent);
        case 4:
        case "end":
          return _context56.stop();
      }
    }, _callee56, this);
  }));
  return function (_x42) {
    return _ref112.apply(this, arguments);
  };
}();